package com.ninegag.android.library.upload

import android.Manifest
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import androidx.core.content.ContextCompat
import com.facebook.common.media.MediaUtils
import com.ninegag.android.library.upload.model.MediaMeta
import com.under9.android.lib.util.file.FileUtil
import com.under9.android.lib.util.MediaUtil
import com.under9.android.lib.util.ThreadUtil
import timber.log.Timber
import java.io.File
import java.util.*

object Util {
    private const val PRELOAD_COOLDOWN = 30 * 1000.toLong()
    const val PRELOAD_LIMIT = 20

    @JvmField val mediaInfoCache = ArrayList<MediaMeta>()
    @Volatile private var lastMediaLoadedTime: Long = 0

    @JvmStatic
    fun generateUploadId(): String {
        return UUID.randomUUID().toString()
    }

    @JvmStatic
    fun getStepString(action: Int): String {
        if (action == Constant.ACTION_IMAGE) {
            return Constant.ACTION_IMAGE_STRING
        }
        if (action == Constant.ACTION_META) {
            return Constant.ACTION_META_STRING
        }
        if (action == Constant.ACTION_CREATION) {
            return Constant.ACTION_CREATION_STRING
        }
        return if (action == Constant.ACTION_ARTICLE_DATA) {
            Constant.ACTION_ARTICLE_DATA_STRING
        } else ""
    }

    // intent
    @JvmStatic
    fun fillRetryIntent(intent: Intent, uploadId: String?): Intent {
        intent.putExtra(Constant.COMMAND, Constant.ACTION_RETRY)
        intent.putExtra(Constant.UPLOAD_ID, uploadId)
        return intent
    }

    /**
     * Preload a set of images to be used by [UploadSourceChooserDialog]
     *
     * @param context applicationContext
     * @param excludedTypes set of supported media types, see below
     *
     * @see MediaMeta.MEDIA_TYPE_GIF
     * @see MediaMeta.MEDIA_TYPE_MP4
     * @see MediaMeta.MEDIA_TYPE_IMAGE
     */
    @JvmStatic
    fun preloadMediaWithContentResolver(context: Context, excludedTypes: Set<Int>? = null) {
        if (System.currentTimeMillis() - lastMediaLoadedTime < PRELOAD_COOLDOWN) return
        if (ContextCompat.checkSelfPermission(context.applicationContext, Manifest.permission.READ_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) return

        FileUtil.triggerMediaScanner(context, null)

        ThreadUtil.getThreadPool().submit {
            synchronized(mediaInfoCache) {
                val mediaInfos: MutableList<MediaMeta> = ArrayList()
                FileUtil.getMediaByContentResolver2(context, PRELOAD_LIMIT, object: FileUtil.LoadMediaCallback {
                    override fun onMediaLoaded(url: String?, duration: Long, mimeType: String?, name: String) {
                        Timber.d("onMediaLoaded, v2 url=$url, mimeType=$mimeType")
                        var mediaInfo: MediaMeta? = null
                        val mediaType = when {
                            MediaUtils.isVideo(mimeType) -> MediaMeta.MEDIA_TYPE_MP4
                            MediaUtil.isGif(mimeType) -> MediaMeta.MEDIA_TYPE_GIF
                            else -> MediaMeta.MEDIA_TYPE_IMAGE
                        }

                        if (excludedTypes?.contains(mediaType) == true) {
                            return
                        }

                        if (MediaUtils.isVideo(mimeType)) {
                            mediaInfo = MediaMeta.newBuilder(MediaMeta.MEDIA_TYPE_MP4)
                                    .contentUri(url)
                                    .videoDuration(duration)
                                    .build()
                        } else {
                            mediaInfo = if (MediaUtil.isGif(mimeType)) {
                                MediaMeta.newBuilder(MediaMeta.MEDIA_TYPE_GIF)
                                        .contentUri(url)
                                        .build()
                            } else {
                                MediaMeta.newBuilder(MediaMeta.MEDIA_TYPE_IMAGE)
                                        .contentUri(url)
                                        .build()
                            }
                        }
                        if (mediaInfo?.contentUri != null && mediaInfo.contentUri.isNotEmpty()) {
                            mediaInfos.add(mediaInfo)
                        }
                    }
                })
                mediaInfos.clear()
                mediaInfos.addAll(mediaInfos)
                Timber.d("loadPhotos() called with: mImageUrls = [$mediaInfoCache]")
            }
        }
    }

    @JvmStatic
    fun createTmpFile(dir: String?, imageType: Int): String? {
        var ext = "jpg"
        if (imageType == BaseUploadSourceActivity.IMAGE_TYPE_GIF) {
            ext = "gif"
        }
        if (imageType == BaseUploadSourceActivity.IMAGE_TYPE_MP4) {
            ext = "mp4"
        }
        val path = String.format("%s%s%s.$ext", dir, File.separator, System.currentTimeMillis())
        Timber.d("createTmpFile: $path")
        return path
    }
}