package com.ninegag.android.library.upload.media

import android.content.Context
import android.net.Uri

import com.ninegag.android.library.upload.media.processor.BaseProcessor
import com.ninegag.android.library.upload.media.processor.GifProcessor
import com.ninegag.android.library.upload.media.processor.ImageProcessor
import com.ninegag.android.library.upload.media.processor.Mp4Processor
import com.ninegag.android.library.upload.media.validator.BaseMediaValidator
import com.ninegag.android.library.upload.media.validator.DefaultImageValidatorConfig
import com.ninegag.android.library.upload.media.validator.GifValidatorConfig
import com.ninegag.android.library.upload.media.validator.ImageValidator
import com.ninegag.android.library.upload.media.validator.BaseValidatorConfig
import com.ninegag.android.library.upload.media.validator.Mp4Validator
import com.ninegag.android.library.upload.media.validator.Mp4ValidatorConfig
import com.ninegag.android.library.upload.model.MediaMeta
import com.under9.android.lib.util.file.SourceFileController
import timber.log.Timber

import java.io.File

/**
 * Created by raymond on 9/5/2017.
 */

open class MediaProcessor(context: Context,
                          private val sourceFileController: SourceFileController,
                          private val mediaValidatorCallback: BaseMediaValidator.MediaValidatorCallback,
                          private val saveMediaCallback: BaseProcessor.SaveMediaCallback,
                          private val shouldCompress: Boolean = true) {

    private val context: Context = context.applicationContext
    private var baseProcessor: BaseProcessor? = null
    private var baseMediaValidator: BaseMediaValidator? = null

    /**
     * Process the meta
     * @param filePath Absolute file path of the media
     * @param tmpFilePath The temporary file path that the media will be saved to. (a cache path)
     */
    @Deprecated("Should use processMediaByContentUri()")
    fun processMediaMetaByAbsoluteFilePath(filePath: String, tmpFilePath: String) {
        @MediaMeta.MediaType val mediaType = getMediaTypeByAbsolutePath(filePath)
        val config = createValidatorConfigByType(mediaType)

        val f = File(filePath)
        baseMediaValidator = createMediaValidatorByType(mediaType, config, mediaValidatorCallback)
        baseProcessor = createProcessorByType(context, sourceFileController, saveMediaCallback, baseMediaValidator, mediaType)

        val mediaMeta = baseProcessor!!.getMediaMeta(f)
        baseProcessor!!.saveMediaToTempFile(mediaMeta, tmpFilePath)
    }

    /**
     * Process media by it's contentUri returned
     */
    fun processMediaByContentUri(contentUri: Uri) {
        @MediaMeta.MediaType val mediaType = getMediaTypeByContentUri(context, contentUri)
        val config = createValidatorConfigByType(mediaType)

        baseMediaValidator = createMediaValidatorByType(mediaType, config, mediaValidatorCallback)
        baseProcessor = createProcessorByType(context, sourceFileController, saveMediaCallback, baseMediaValidator, mediaType)

        baseProcessor!!.saveMediaToTempFile(contentUri)
    }

    /**
     * To
     * @param mediaType
     * @return
     */
    protected open fun createValidatorConfigByType(@MediaMeta.MediaType mediaType: Int): BaseValidatorConfig {
        return when (mediaType) {
            MediaMeta.MEDIA_TYPE_IMAGE -> DefaultImageValidatorConfig()
            MediaMeta.MEDIA_TYPE_GIF -> GifValidatorConfig()
            MediaMeta.MEDIA_TYPE_MP4 -> Mp4ValidatorConfig()
            else -> DefaultImageValidatorConfig()
        }
    }

    /**
     *
     * @param mediaType
     * @param config
     * @param callback
     * @return
     */
    protected fun createMediaValidatorByType(@MediaMeta.MediaType mediaType: Int,
                                             config: BaseValidatorConfig,
                                             callback: BaseMediaValidator.MediaValidatorCallback): BaseMediaValidator? {
        return when (mediaType) {
            MediaMeta.MEDIA_TYPE_IMAGE, MediaMeta.MEDIA_TYPE_GIF -> ImageValidator(config, callback)
            MediaMeta.MEDIA_TYPE_MP4 -> Mp4Validator(config, callback)
            else -> null
        }
    }

    /**
     *
     *
     * @param context
     * @param validator
     * @param mediaType
     * @return
     */
    protected open fun createProcessorByType(context: Context,
                                             sourceFileController: SourceFileController,
                                             saveMediaCallback: BaseProcessor.SaveMediaCallback, validator: BaseMediaValidator?, @MediaMeta.MediaType mediaType: Int): BaseProcessor {
        return when (mediaType) {
            MediaMeta.MEDIA_TYPE_IMAGE -> ImageProcessor(context, sourceFileController, saveMediaCallback, validator!!, shouldCompress)
            MediaMeta.MEDIA_TYPE_GIF -> GifProcessor(context, sourceFileController, saveMediaCallback, validator)
            MediaMeta.MEDIA_TYPE_MP4 -> Mp4Processor(context, sourceFileController, saveMediaCallback, validator)
            else -> ImageProcessor(context, sourceFileController, saveMediaCallback, validator!!, shouldCompress)
        }
    }

    companion object {
        @JvmStatic
        fun getMediaTypeByContentUri(context: Context, contentUri: Uri): Int {
            return when (context.applicationContext.contentResolver.getType(contentUri) ?: "") {
                "image/gif" -> MediaMeta.MEDIA_TYPE_GIF
                "video/mp4" -> MediaMeta.MEDIA_TYPE_MP4
                else -> MediaMeta.MEDIA_TYPE_IMAGE
            }
        }

        @JvmStatic
        fun getMediaTypeByAbsolutePath(filePath: String?): Int {
            // Fallback to image
            if (filePath == null) return MediaMeta.MEDIA_TYPE_IMAGE

            return try {
                val index = filePath.lastIndexOf(".")
                val ext = filePath.substring(index, filePath.length)
                when (ext.toLowerCase()) {
                    ".gif" -> MediaMeta.MEDIA_TYPE_GIF
                    ".mp4" -> MediaMeta.MEDIA_TYPE_MP4
                    else -> MediaMeta.MEDIA_TYPE_IMAGE
                }
            } catch (e: StringIndexOutOfBoundsException) {
                Timber.e(e)
                MediaMeta.MEDIA_TYPE_IMAGE
            }
        }
    }
}
