package com.ninegag.android.library.upload.media.processor

import android.content.Context
import android.net.Uri
import android.os.AsyncTask
import android.os.ParcelFileDescriptor
import androidx.annotation.UiThread
import androidx.annotation.WorkerThread
import com.ninegag.android.library.upload.media.validator.BaseMediaValidator
import com.ninegag.android.library.upload.model.MediaMeta
import com.under9.android.lib.util.file.SourceFileController
import io.reactivex.Single
import io.reactivex.SingleOnSubscribe
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.io.File

/**
 * Created by raymond on 9/5/2017.
 */
abstract class BaseProcessor(context: Context,
                             @JvmField var sourceFileController: SourceFileController,
                             private val saveMediaCallback: SaveMediaCallback,
                             @JvmField var mediaValidator: BaseMediaValidator) {
    interface SaveMediaCallback {
        /**
         * Called when media save to tmp file location successfully
         * @param mediaMeta The original media meta info
         * @param tmpFileLocation The tmp file location after media is being processed, which serve as a cache copy
         */
        @UiThread
        fun onMediaSaveSuccess(mediaMeta: MediaMeta, tmpFileLocation: String?)

        /**
         *
         */
        @UiThread
        fun onMediaSaveProgress()

        /**
         * Called when media save to tmp file location failed
         * @param mediaMeta The original media meta info
         * @param tmpFileLocation File location where it is saved
         * @param e
         */
        @UiThread
        fun onMediaSaveFailed(mediaMeta: MediaMeta?, tmpFileLocation: String?, e: Throwable?)
    }

    val context: Context = context.applicationContext
    /**
     * TODO: 7/9/2017 Probably we should move this function to a new class called somehting like MetaDataRetriever
     *
     * @deprecated
     * Get all the necessary media info, e.g. width, height, file size and type
     * @param f Input file
     * @return Meida's info
     */
    abstract fun getMediaMeta(f: File): MediaMeta

    /**
     *
     */
    fun saveMediaToTempFile(contentUri: Uri) {
        saveMediaCallback.onMediaSaveProgress()

        val d = Single.create(SingleOnSubscribe<MediaMeta> { emitter ->
            context.contentResolver.openFileDescriptor(contentUri, "r")?.use {
                val tmpFilePath = sourceFileController.uploadDir(context) + File.separatorChar + "tmp_${System.currentTimeMillis()}"
                try {
                    val mediaMeta = saveMedia(it, contentUri, tmpFilePath)
                    emitter.onSuccess(mediaMeta)
                } catch (e: Exception) {
                    emitter.onError(e)
                }
            }
        }).subscribeOn(Schedulers.io()).observeOn(AndroidSchedulers.mainThread()).subscribe { mediaMeta, throwable ->
            if (throwable == null) {
                val originalFileSize = mediaValidator.validateFileSize(mediaMeta!!)
                val originalFileDimension = mediaValidator.validateFileDimension(mediaMeta)
                if (originalFileSize && originalFileDimension) {
                    saveMediaCallback.onMediaSaveSuccess(mediaMeta, mediaMeta.filePath)
                } else {
                    val compressedFileSize = mediaValidator.validateFileSize(mediaMeta)
                    val compressedFileDimension = mediaValidator.validateFileDimension(mediaMeta)
                    if (compressedFileSize && compressedFileDimension) {
                        saveMediaCallback.onMediaSaveSuccess(mediaMeta, mediaMeta.filePath)
                    }
                }
            } else {
                saveMediaCallback.onMediaSaveFailed(mediaMeta, "", throwable)
            }
        }
    }

    @WorkerThread
    @Throws(Exception::class)
    protected abstract fun saveMedia(parcelFileDescriptor: ParcelFileDescriptor, contentUri: Uri, tmpFileLocation: String): MediaMeta

    /**
     * @param mediaMeta
     * @param tmpFileLocation
     */
    @Deprecated("Should use contentUri instead\n" + "      Save to a temp file asynchronously\n" + "      ")
    fun saveMediaToTempFile(mediaMeta: MediaMeta, tmpFileLocation: String) {
        object : AsyncTask<Void?, Any?, Any?>() {
            var exception: Exception? = null
            override fun onPreExecute() {
                super.onPreExecute()
                publishProgress(null)
            }

            protected override fun doInBackground(vararg params: Void?): Void? {
                try {
                    saveMedia(mediaMeta, tmpFileLocation)
                } catch (e: Exception) {
                    exception = e
                }
                return null
            }

            protected override fun onProgressUpdate(vararg values: Any?) {
                super.onProgressUpdate(*values)
                saveMediaCallback.onMediaSaveProgress()
            }

            override fun onPostExecute(o: Any?) {
                if (exception != null) {
                    saveMediaCallback.onMediaSaveFailed(mediaMeta, tmpFileLocation, exception)
                } else {
                    val originalFileSize = mediaValidator.validateFileSize(mediaMeta!!)
                    val originalFileDimension = mediaValidator.validateFileDimension(mediaMeta)
                    if (originalFileSize && originalFileDimension) {
                        saveMediaCallback.onMediaSaveSuccess(mediaMeta, tmpFileLocation)
                    } else {
                        val processedMediaMeta = getMediaMeta(File(tmpFileLocation))
                        val compressedFileSize = mediaValidator.validateFileSize(processedMediaMeta)
                        val compressedFileDimension = mediaValidator.validateFileDimension(processedMediaMeta)
                        if (compressedFileSize && compressedFileDimension) {
                            saveMediaCallback.onMediaSaveSuccess(mediaMeta, tmpFileLocation)
                        }
                    }
                }
            }
        }.executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR)
    }

    @WorkerThread
    @Throws(Exception::class)
    @Deprecated("Should use inputStream's version")
    protected abstract fun saveMedia(mediaMeta: MediaMeta, tmpFileLocation: String)

}