package com.ninegag.android.library.upload.media.processor

import android.content.Context
import android.graphics.Bitmap
import android.media.MediaMetadataRetriever
import android.media.ThumbnailUtils
import android.net.Uri
import android.os.Build
import android.os.CancellationSignal
import android.os.ParcelFileDescriptor
import android.provider.MediaStore
import android.util.Log
import android.util.Size
import com.iceteck.silicompressorr.SiliCompressor
import com.iceteck.silicompressorr.videocompression.VideoCompressConfig
import com.ninegag.android.library.upload.media.validator.BaseMediaValidator
import com.ninegag.android.library.upload.model.MediaMeta
import com.under9.android.lib.util.file.SourceFileController
import java.io.File
import java.io.FileInputStream
import java.io.FileOutputStream

/**
 * Created by raymond on 9/5/2017.
 */
class Mp4Processor(context: Context?,
                   sourceFileController: SourceFileController?,
                   saveMediaCallback: SaveMediaCallback?,
                   mediaValidator: BaseMediaValidator?) : BaseProcessor(context!!, sourceFileController!!, saveMediaCallback!!, mediaValidator!!) {
    private var path: String? = null
    private var mp4Height = 0
    private var mp4Width = 0
    private var enableCompression = false

    override fun getMediaMeta(f: File): MediaMeta {
        var mp4Duration: Long = 0
        var hasAudio = false
        try {
            val retriever = MediaMetadataRetriever()
            path = Uri.fromFile(f).path
            retriever.setDataSource(path)
            if (retriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_VIDEO_ROTATION).toInt() % 180 == 90) {
                mp4Height = retriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_VIDEO_WIDTH).toInt()
                mp4Width = retriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_VIDEO_HEIGHT).toInt()
            } else {
                mp4Width = retriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_VIDEO_WIDTH).toInt()
                mp4Height = retriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_VIDEO_HEIGHT).toInt()
            }
            mp4Duration = retriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_DURATION).toLong()
            val hasAudioStr = retriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_HAS_AUDIO)
            if (hasAudioStr != null) {
                hasAudio = hasAudioStr == "yes"
            }
            Log.d(TAG, "getMediaMeta: h=$mp4Height, w=$mp4Width")
        } catch (e: Exception) {
            Log.e(TAG, "getMediaMeta: ", e)
        }
        return MediaMeta.newBuilder(MediaMeta.MEDIA_TYPE_MP4)
                .filePath(f.absolutePath)
                .fileSize(f.length())
                .widthAndHeight(mp4Width, mp4Height)
                .videoDuration(mp4Duration)
                .hasAudio(hasAudio)
                .build()
    }

    @Throws(Exception::class)
    override fun saveMedia(mediaMeta: MediaMeta, tmpFileLocation: String) {
        var path = mediaMeta.filePath
        // Compress video only if SDK int is > 18
        // Compress video only if file size is greater than config
        if (enableCompression && Build.VERSION.SDK_INT > Build.VERSION_CODES.JELLY_BEAN_MR2 && mediaMeta.fileSize > mediaValidator.baseValidatorConfig.fileSizeLimit) {
            val lastSlashIndex = tmpFileLocation.lastIndexOf("/")
            val outputPath = tmpFileLocation.substring(0, lastSlashIndex)
            var compressConfig: VideoCompressConfig? = null
            var outWidth = 0
            var outHeight = 0
            val isLandscape = mp4Width > mp4Height
            val hwRatio = mp4Height / (mp4Width + 0f)
            if (mp4Width > MAX_MP4_DIMEN) {
                if (isLandscape) {
                    outWidth = MAX_MP4_DIMEN
                    outHeight = (hwRatio * outWidth).toInt()
                    // https://stackoverflow.com/questions/36915383/what-does-error-code-1010-in-android-mediacodec-mean
                    // value must be divisible by 2
                    if (outHeight % 2 != 0) {
                        outHeight += 1
                    }
                } else {
                    outHeight = MAX_MP4_DIMEN
                    outWidth = (outHeight / hwRatio).toInt()
                    if (outWidth % 2 != 0) {
                        outWidth += 1
                    }
                    // We do a swap for height and width here is because inside compressVideo() call,
                    // it will detect rotation of the original video and apply the rotation correctly
                    val tmp: Int
                    tmp = outHeight
                    outHeight = outWidth
                    outWidth = tmp
                }
                compressConfig = VideoCompressConfig.newConfig(outWidth, outHeight, path, outputPath)
            }
            if (compressConfig != null) {
                val compressedVidPath = SiliCompressor.with(context)
                        .compressVideo(compressConfig)
                val newFile = File(compressedVidPath)
                // Only use the compressed video only if the compressed size is smaller than original
                // Though it is not likely
                if (newFile.length() < mediaMeta.fileSize) {
                    path = compressedVidPath
                }
            }
        }
        val fos = FileOutputStream(tmpFileLocation)
        val fis = FileInputStream(path)
        val buf = ByteArray(1024)
        var len: Int
        while (fis.read(buf).also { len = it } > 0) {
            fos.write(buf, 0, len)
        }
        fis.close()
        fos.close()
        var bitmap: Bitmap?
        bitmap = if (Build.VERSION.SDK_INT <= 28) {
            ThumbnailUtils.createVideoThumbnail(path, MediaStore.Images.Thumbnails.MINI_KIND)
        } else {
            ThumbnailUtils.createVideoThumbnail(File(path), Size(mp4Width, mp4Height), CancellationSignal())
        }
        val outputPath = tmpFileLocation + "_preview.jpg"
        val cachePreviewFile = File(outputPath)
        val bitmapFos = FileOutputStream(cachePreviewFile)
        bitmap!!.compress(Bitmap.CompressFormat.JPEG, 100, bitmapFos)
        bitmap.recycle()
        bitmap = null
    }

    /**
     * To enable or disable video compresion
     * @param enableCompression
     */
    fun setEnableCompression(enableCompression: Boolean) {
        this.enableCompression = enableCompression
    }

    @Throws(Exception::class)
    override fun saveMedia(parcelFileDescriptor: ParcelFileDescriptor, contentUri: Uri, tmpFileLocation: String): MediaMeta {
        val mp4Height: Int
        val mp4Width: Int
        var mp4Duration: Long = 0
        var hasAudio = false

        val fos = FileOutputStream(tmpFileLocation)
        val fis = FileInputStream(parcelFileDescriptor.fileDescriptor)
        var fileSize = fis.channel.size()
        val retriever = MediaMetadataRetriever()
        retriever.setDataSource(context, contentUri)

        if (retriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_VIDEO_ROTATION).toInt() % 180 == 90) {
            mp4Height = retriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_VIDEO_WIDTH).toInt()
            mp4Width = retriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_VIDEO_HEIGHT).toInt()
        } else {
            mp4Width = retriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_VIDEO_WIDTH).toInt()
            mp4Height = retriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_VIDEO_HEIGHT).toInt()
        }

        mp4Duration = retriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_DURATION).toLong()
        val hasAudioStr = retriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_HAS_AUDIO)
        if (hasAudioStr != null) {
            hasAudio = hasAudioStr == "yes"
        }

        val buf = ByteArray(1024)
        var len: Int
        while (fis.read(buf).also { len = it } > 0) {
            fos.write(buf, 0, len)
        }
        fis.close()
        fos.close()

        var finalPath = tmpFileLocation
        if (enableCompression && fileSize > mediaValidator.baseValidatorConfig.fileSizeLimit) {
            val lastSlashIndex = tmpFileLocation.lastIndexOf("/")
            val outputPath = tmpFileLocation.substring(0, lastSlashIndex)
            var compressConfig: VideoCompressConfig? = null
            var outWidth = 0
            var outHeight = 0
            val isLandscape = mp4Width > mp4Height
            val hwRatio = mp4Height / (mp4Width + 0f)
            if (mp4Width > MAX_MP4_DIMEN) {
                if (isLandscape) {
                    outWidth = MAX_MP4_DIMEN
                    outHeight = (hwRatio * outWidth).toInt()
                    // https://stackoverflow.com/questions/36915383/what-does-error-code-1010-in-android-mediacodec-mean
                    // value must be divisible by 2
                    if (outHeight % 2 != 0) {
                        outHeight += 1
                    }
                } else {
                    outHeight = MAX_MP4_DIMEN
                    outWidth = (outHeight / hwRatio).toInt()
                    if (outWidth % 2 != 0) {
                        outWidth += 1
                    }
                    // We do a swap for height and width here is because inside compressVideo() call,
                    // it will detect rotation of the original video and apply the rotation correctly
                    val tmp: Int
                    tmp = outHeight
                    outHeight = outWidth
                    outWidth = tmp
                }
                compressConfig = VideoCompressConfig.newConfig(outWidth, outHeight, tmpFileLocation, outputPath)
            }
            if (compressConfig != null) {
                val compressedVidPath = SiliCompressor.with(context)
                        .compressVideo(compressConfig)
                val newFile = File(compressedVidPath)
                // Only use the compressed video only if the compressed size is smaller than original
                // Though it is not likely
                if (newFile.length() < fileSize) {
                    finalPath = compressedVidPath
                    fileSize = newFile.length()
                    // Clean up the uncompressed one
                    File(tmpFileLocation).delete()
                }
            }
        }



        var bitmap: Bitmap?
        bitmap = if (Build.VERSION.SDK_INT <= 28) {
            ThumbnailUtils.createVideoThumbnail(finalPath, MediaStore.Images.Thumbnails.MINI_KIND)
        } else {
            ThumbnailUtils.createVideoThumbnail(File(finalPath), Size(mp4Width, mp4Height), CancellationSignal())
        }
        val outputPath = finalPath + PREVIEW_THUMBNAIL_SUFFIX
        val cachePreviewFile = File(outputPath)
        val bitmapFos = FileOutputStream(cachePreviewFile)
        bitmap!!.compress(Bitmap.CompressFormat.JPEG, 100, bitmapFos)
        bitmap.recycle()
        bitmap = null

        return MediaMeta.newBuilder(MediaMeta.MEDIA_TYPE_MP4)
                .filePath(finalPath)
                .fileSize(fileSize)
                .widthAndHeight(mp4Width, mp4Height)
                .videoDuration(mp4Duration / 1000)
                .hasAudio(hasAudio)
                .build()
    }

    companion object {
        private const val MAX_MP4_DIMEN = 480 //TODO: may be we should put it to a config
        private const val TAG = "Mp4Processor"
        private const val DEBUG = false
        const val PREVIEW_THUMBNAIL_SUFFIX = "_preview.jpg"
    }
}