package com.under9.android.comments.data.repository

import androidx.collection.ArrayMap
import androidx.collection.ArraySet
import com.google.gson.reflect.TypeToken
import com.under9.android.comments.GsonFactory
import com.under9.android.comments.controller.DataController
import com.under9.android.comments.data.query.comment.CommentListQueryParam2
import com.under9.android.comments.data.query.comment.CommentListQueryResult2
import com.under9.android.comments.model.*
import com.under9.android.comments.model.api.ApiComment
import de.greenrobot.dao.query.QueryBuilder
import timber.log.Timber
import kotlin.collections.ArrayList

class LocalCommentListRepository(
        private val dataController: DataController,
        private val localUserRepository: LocalUserRepositoryInterface
): LocalCommentListRepositoryInterface {

    override fun getMaxLocalInsertOrder(listKey: String): Long {
        val cursor = dataController.session
                .database.rawQuery("SELECT MAX(${CommentListItemDao.Properties.LocalInsertOrder.columnName}) FROM ${CommentListItemDao.TABLENAME} WHERE ${CommentListItemDao.Properties.ListKey.columnName} = ?", arrayOf(listKey))
        var startingMaxId = 0L
        if (cursor.moveToFirst()) {
            startingMaxId = cursor.getLong(0)
        }
        return startingMaxId
    }

    override fun updateLocalInsertOrder(listKey: String, startingOrder: Int, prependSize: Int) {
        val lazyList = dataController.commentListItemDao
                .queryBuilder()
                .where(CommentListItemDao.Properties.LocalInsertOrder.ge(startingOrder))
                .where(CommentListItemDao.Properties.ListKey.eq(listKey))
                .orderAsc(CommentListItemDao.Properties.LocalInsertOrder)
                .listLazy()

        for (i in 0 until lazyList.size) {
            lazyList[i].localInsertOrder = lazyList[i].localInsertOrder + prependSize
            lazyList[i].update()
        }
    }

    override fun obtainListByListKey(listKey: String): CommentList {
        val builder = dataController.commentListDao.queryBuilder()
                .where(CommentListDao.Properties.ListKey.eq(listKey))

        val list = builder.build().list()
        return if (list.size == 0) {
            val commentList = CommentList()
            commentList.listKey = listKey
            commentList.prevCursor = GsonFactory.getGson().toJson(listOf<String?>())
            commentList.nextCursor = GsonFactory.getGson().toJson(listOf<String?>())
            commentList.hasNext = false
            commentList.hasPrev = false
            commentList.level = 0
            commentList.opUserId = ""
            dataController.commentListDao.insert(commentList)
            commentList
        } else {
            list[0]
        }
    }

    override fun getListMeta(listKey: String, direction: Int): Map<String, String?> {
        val list = obtainListByListKey(listKey)
        val token = object : TypeToken<ArrayList<String>>(){}.type
        val map = androidx.collection.ArrayMap<String, String?>()

        return if (direction == CommentListQueryParam2.DIRECTION_NEXT) {
            val nextList: ArrayList<String?> = GsonFactory.getGson().fromJson(list.nextCursor, token)
            val prevList: ArrayList<String?> = GsonFactory.getGson().fromJson(list.prevCursor, token)

            Timber.tag("CommentListWrapper").d("nextList=$nextList, prevList=$prevList, direction=$direction")

            val next = if (nextList.isEmpty()) null else nextList[nextList.size - 1]
            val prev = if (prevList.isEmpty()) null else prevList[nextList.size - 1]

            map[CommentListQueryResult2.KEY_PREV] = prev
            map[CommentListQueryResult2.KEY_NEXT] = next
            map[CommentListQueryResult2.KEY_OP_USER_ID] = list.opUserId
            map[CommentListQueryResult2.KEY_LEVEL] = list.level.toString()

            map
        } else {
            val nextList: ArrayList<String?> = GsonFactory.getGson().fromJson(list.nextCursor, token)
            val prevList: ArrayList<String?> = GsonFactory.getGson().fromJson(list.prevCursor, token)

            Timber.tag("CommentListWrapper").d("nextList=$nextList, prevList=$prevList, direction=$direction")

            val prev = if (prevList.isEmpty()) null else prevList[0]
            val next = if (nextList.isEmpty()) null else nextList[0]

            map[CommentListQueryResult2.KEY_PREV] = prev
            map[CommentListQueryResult2.KEY_NEXT] = next
            map[CommentListQueryResult2.KEY_OP_USER_ID] = list.opUserId
            map[CommentListQueryResult2.KEY_LEVEL] = list.level.toString()

            map
        }
    }

    override fun updateListByListKey(listKey: String, prevCursor: String?, nextCursor: String?, opUserId: String, level: Int, direction: Int, lastRefreshTs: Long) {
        val token = object : TypeToken<ArrayList<String?>>(){}.type
        val list = obtainListByListKey(listKey)
        // We have to store all the pagination information such that there won't be missing pages when
        // navigation through next and previous
        val prevList: ArrayList<String?> = GsonFactory.getGson().fromJson(list.prevCursor, token)
        val nextList: ArrayList<String?> = GsonFactory.getGson().fromJson(list.nextCursor, token)

        val prevSet = ArraySet(prevList)
        val nextSet = ArraySet(nextList)

        Timber.tag("CommentListWrapper").d("updateListByListKey list=$nextList, prevList=$prevList")

        if (direction == CommentListQueryParam2.DIRECTION_NEXT) {
            if (!prevSet.contains(prevCursor)) prevList.add(prevCursor)
            if (!nextSet.contains(nextCursor)) nextList.add(nextCursor)
        } else if (direction == CommentListQueryParam2.DIRECTION_PREV) {
            if (!prevSet.contains(prevCursor)) prevList.add(0, prevCursor)
            if (!nextSet.contains(nextCursor)) nextList.add(0, nextCursor)
        } else if (direction == -1) { // Clear list
            prevList.clear()
            nextList.clear()
        }

        list.apply {
            this.prevCursor = GsonFactory.getGson().toJson(prevList)
            this.nextCursor = GsonFactory.getGson().toJson(nextList)
            this.hasNext = nextCursor != null
            this.hasPrev = prevCursor != null
            this.opUserId = opUserId
            this.level = level
            if (lastRefreshTs > 0) {
                this.lastFetch = lastRefreshTs
            }
            this.updateTs = System.currentTimeMillis()
        }
        dataController.commentListDao.update(list)
    }

    override fun getCommentListItem(listKey: String): List<CommentListItem> {
        return dataController.commentListItemDao.queryBuilder()
                .where(CommentListItemDao.Properties.ListKey.eq(listKey))
                .listLazy()
    }

    override fun getCommentItemById(commentId: String): CommentItem? {
        return dataController.commentItemDao.queryBuilder()
                .where(CommentItemDao.Properties.CommentId.eq(commentId))
                .build().unique()
    }

    override fun getCommentItemByDbId(dbId: Long): CommentItem? {
        return dataController.commentItemDao.queryBuilder()
                .where(CommentItemDao.Properties.Id.eq(dbId))
                .build().unique()
    }

    override fun getCommentItemsByIds(ids: Collection<String>): List<CommentItem> {
        return dataController.commentItemDao.queryBuilder()
                .where(CommentItemDao.Properties.CommentId.`in`(ids))
                .build().list()
    }

    override fun getCommentListItemsByIds(ids: Collection<String>): List<CommentListItem> {

        for (i in ids) {
            val l = dataController.commentListItemDao.queryBuilder()
//                    .where(CommentListItemDao.Properties.CommentId.eq(i))
                    .build()
                    .list()

            Timber.d("getCommentListItemsByIds=$l")
        }

        return dataController.commentListItemDao.queryBuilder()
                .where(CommentListItemDao.Properties.CommentId.`in`(ids))
                .build().list()
    }

    override fun getLatestComment(listKey: String): CommentItem? {
        return dataController.commentListItemDao.queryBuilder()
                .where(CommentListItemDao.Properties.ListKey.eq(listKey))
                .orderDesc(CommentListItemDao.Properties.LocalInsertOrder)
                .limit(1).unique()?.commentItem

    }

    override fun replacePendingCommentItem(
            url: String, id: Long, item: ApiComment, commentUser: User): CommentItem? {
        val commentListItem = getCommentListItemByDbId(id)
        val commentItem = commentListItem?.commentItem ?: return null
        applyCommentItem(url, commentItem, item, commentUser)
        dataController.commentItemDao.updateInTx(commentItem)
        return commentItem
    }

    override fun insertOrUpdateCommentItemBatches(
            insertCommentItemBatches: List<CommentItem>,
            updateCommentItemBatches: List<CommentItem>,
            insertCommentListItemBatches: List<CommentListItem>,
            updateCommentListItemBatches: List<CommentListItem>
    ) {
        dataController.session.runInTx {
            val itemMapping = ArrayMap<String, CommentItem>()
            // Insert all new CommentItems
            if (insertCommentItemBatches.isNotEmpty()) {
                dataController.commentItemDao.insertInTx(insertCommentItemBatches)
                for (item in insertCommentItemBatches) {
                    itemMapping[item.commentId] = item
                }
            }

            // Update all new CommentItems
            if (updateCommentItemBatches.isNotEmpty()) {
                dataController.commentItemDao.updateInTx(updateCommentItemBatches)
                for (item in updateCommentItemBatches) {
                    itemMapping[item.commentId] = item
                }
            }

            // Link CommentItem to CommentListItem
            if (insertCommentListItemBatches.isNotEmpty()) {
                for (item in insertCommentListItemBatches) {
                    if (itemMapping.contains(item.commentId)) {
                        // Reassign here to fetch the newly inserted comment database id (local)
                        item.commentItem = itemMapping[item.commentId]!!
                    }
                }
                dataController.commentListItemDao.insertInTx(insertCommentListItemBatches)
            }

            // Link CommentItem to CommentListItem
            if (updateCommentListItemBatches.isNotEmpty()) {
                for (item in updateCommentListItemBatches) {
                    if (itemMapping.contains(item.commentId)) {
                        // Reassign here to fetch the newly inserted comment database id (local)
                        item.commentItem = itemMapping[item.commentId]!!
                    }
                }
                dataController.commentListItemDao.updateInTx(updateCommentListItemBatches)
            }
        }
    }

    private fun applyCommentItem(url: String, commentItem: CommentItem, item: ApiComment, commentUser: User): CommentItem? {
        commentItem.apply {
            this.url = url
            commentId = item.commentId
            threadId = item.threadId
            text = item.text
            timestamp = item.timestamp
            mentionMapping = GsonFactory.getGson().toJson(item.mentionMapping)
            type = item.type
            permalink = item.permalink
            level = item.level
            isVoteMasked = item.isVoteMasked
            mediaText = item.mediaText
            likeCount = item.likeCount
            dislikeCount = item.dislikeCount
            childrenTotal = item.childrenTotal
            childrenUrl = item.childrenUrl
            parent = item.parent
            media = if (item.media != null) GsonFactory.getGson().toJson(item.media) else null
            user = commentUser
        }
        return commentItem
    }

    override fun replaceCommentItem(url: String, item: ApiComment, commentUser: User): CommentItem {
        var commentItem = getCommentItemById(item.commentId)
        Timber.d("updatedItem=${commentItem?.commentId}, text=${commentItem?.mediaText}, likeStatus=${commentItem?.likeStatus}, count=${commentItem?.likeCount}")
        
        var isNew = false
        if (commentItem == null) {
            commentItem = CommentItem()
            commentItem.apply {
                likeCount = 0
                dislikeCount = 0
                likeStatus = CommentItem.LIKE_STATUS_NORMAL
                status = CommentItem.STATUS_NORMAL
            }
            isNew = true
        }

        applyCommentItem(url, commentItem, item, commentUser)

        if (isNew) {
            dataController.commentItemDao.insert(commentItem)
        } else {
            dataController.commentItemDao.update(commentItem)
        }

        return commentItem
    }

    override fun getNextListItems(listKey: String, offset: Int, limit: Int): List<CommentListItem> {
        Timber.tag("CommentListWrapper").v("getNextListItems listKey=$listKey, offset=$offset, limit=$limit")
        val queryBuilder = createListItemQueryBuilder(listKey)
                .where(CommentListItemDao.Properties.Status.notEq(CommentItem.STATUS_REPORTED))
                .where(CommentListItemDao.Properties.Status.notEq(CommentItem.STATUS_DELETED))
                .orderAsc(CommentListItemDao.Properties.LocalInsertOrder)
                .offset(offset)
                .limit(limit)
        return queryBuilder.list()
    }

    override fun replaceParent(listKey: String, url: String, item: ApiComment, commentUser: User) {
        val queryBuilder = createListItemQueryBuilder(listKey)
        val items = queryBuilder.list()
        if (items.isEmpty()) {
            createListItem(0, listKey, url, item, commentUser)
        } else {
            replaceCommentItem(url, item, commentUser)
        }
    }

    override fun updateCommentItems(commentItems: List<CommentItem>) {
        for (i in 0 until commentItems.size) {
            dataController.commentItemDao.update(commentItems[i])
        }
    }

    override fun clearList(listKey: String) {
        dataController.session.runInTx {
            val queryBuilder = createListItemQueryBuilder(listKey)
            queryBuilder
                    .where(CommentListItemDao.Properties.Status.notEq(CommentItem.STATUS_PENDING))
                    .buildDelete()
                    .executeDeleteWithoutDetachingEntities()
            updateListByListKey(listKey, null, null, "", 0, -1, 0)
        }
    }

    override fun getCommentListItemByDbId(id: Long): CommentListItem? {
        val queryBuilder = dataController.commentListItemDao.queryBuilder()
                .where(CommentListItemDao.Properties.Id.eq(id))
        return queryBuilder.unique()
    }

    override fun getCommentListItemByCommentDbId(id: Long): List<CommentListItem> {
        val queryBuilder = dataController.commentListItemDao.queryBuilder()
                .where(CommentListItemDao.Properties.CommentDBId.eq(id))
        return queryBuilder.listLazy()
    }

    override fun insertOrUpdateFromApiComments(queryParam: CommentListQueryParam2,
                                               apiCommentListLevel: Int,
                                               apiComments: List<ApiComment>,
                                               userMapping: Map<String, User>) {

        val commentIds = apiComments.map { it.commentId }
        val localCommentItems = getCommentItemsByIds(commentIds)
        val commentIdMapping = ArrayMap<String, CommentItem>()
        localCommentItems.forEach { commentIdMapping[it.commentId] = it }

        var startingOrder = if (queryParam.direction == CommentListQueryParam2.DIRECTION_NEXT) {
            getMaxLocalInsertOrder(queryParam.listKey)
        } else {
            val order = apiCommentListLevel - 1L // level field in API representing if it's loading first level comment or second level comment
            updateLocalInsertOrder(queryParam.listKey, order.toInt(), apiComments.size)
            order
        }
        val newCommentItems = arrayListOf<CommentItem>()
        val newCommentListItems = arrayListOf<CommentListItem>()
        val oldCommentItems = arrayListOf<CommentItem>()

        for (i in apiComments.indices) {
            val apiComment = apiComments[i]
            val updatedComment: CommentItem
            val existingComment: CommentItem? = commentIdMapping[apiComment.commentId]
            val existingUser: User? = userMapping[apiComment.user.userId]

            updatedComment = if (commentIdMapping.contains(apiComment.commentId)) {
                val oldComment = ModelFactory.updateCommentItem(queryParam.url, apiComment, existingComment!!, existingUser!!)
                oldCommentItems.add(oldComment)
                oldComment
            } else {
                val newComment = ModelFactory.createCommentItem(queryParam.url, apiComment, existingUser!!)
                newCommentItems.add(newComment)
                newComment
            }
            newCommentListItems.add(
                    ModelFactory.createOrUpdateCommentListItem(
                            if (queryParam.direction == CommentListQueryParam2.DIRECTION_NEXT) ++startingOrder else startingOrder++,
                            queryParam.listKey, updatedComment, null))
        }

        // Old COMMENT_LIST_ITEM is always empty as it will be deleted everytime after refresh
        insertOrUpdateCommentItemBatches(newCommentItems, oldCommentItems, newCommentListItems, emptyList())
    }

    private fun createListItem(startingOrder: Long, listKey: String, url: String, item: ApiComment, commentUser: User): Boolean {
        var isNew = false

        val commentItem = replaceCommentItem(url, item, commentUser)
        val existingCommentListItem = createListItemQueryBuilder(listKey)
                .where(CommentListItemDao.Properties.CommentDBId.eq(commentItem.id))
                .unique()

        dataController.session.runInTx {

            isNew = existingCommentListItem == null

            val listItem = if (isNew) {
                CommentListItem()
            } else {
                existingCommentListItem
            }

            listItem.apply {
                this.listKey = listKey
                this.commentId = commentItem.commentId
                this.commentItem = commentItem
                this.localInsertOrder = startingOrder
                this.status = if (isNew) CommentItem.STATUS_NORMAL else listItem.status
            }

            if (isNew) {
                dataController.commentListItemDao.insert(listItem)
            } else {
                dataController.commentListItemDao.update(listItem)
            }
        }
        return isNew
    }

    private fun createListItemQueryBuilder(listKey: String): QueryBuilder<CommentListItem> {
        val queryBuilder = dataController.commentListItemDao.queryBuilder()
                .where(CommentListItemDao.Properties.ListKey.eq(listKey))
                .where(CommentListItemDao.Properties.CommentDBId.isNotNull)
        return queryBuilder
    }

    override fun createPendingCommentItem(url: String,
                                          listKey: String, level: Int,
                                          content: String, parent: String?, sourceMetaJson: String?, sourceMetaHash: String?): CommentListItem {
        val loggedInUser = localUserRepository.getUserByUserId(dataController.loggedInUserId)
        val commentItem = CommentItem()
        commentItem.apply {
            this.url = url
            commentId = "local_${System.currentTimeMillis()}"
            threadId = ""
            isFollowed = true
            text = content
            mediaText = content
            timestamp = System.currentTimeMillis()
            mentionMapping = ""
            type = CommentItem.MEDIA_TYPE_TEXT
            permalink = ""
            this.level = level
            isVoteMasked = 0
            mediaText = ""
            likeCount = 0
            dislikeCount = 0
            likeStatus = CommentItem.LIKE_STATUS_NORMAL
            childrenTotal = 0
            status = CommentItem.STATUS_PENDING
            childrenUrl = ""
            this.parent = parent
            media = ""
            user = loggedInUser
        }

        val commentListItem = CommentListItem()


        dataController.session.runInTx {
            val id = dataController.commentItemDao.insert(commentItem)
            commentListItem.apply {
                this.commentDBId = id
                this.commentId = commentItem.commentId
                this.listKey = listKey
                this.status = CommentItem.STATUS_PENDING
                this.commentItem = commentItem
                this.sourceMetaHash = sourceMetaHash
                this.sourceMetaJson = sourceMetaJson
                this.localInsertOrder = getMaxLocalInsertOrder(listKey) + 1
            }
            dataController.commentListItemDao.insert(commentListItem)
        }

        return commentListItem
    }

    override fun clearPendingItemByDbId(id: Long) {
        val commentListItem = dataController.commentListItemDao.queryBuilder()
                .where(CommentListItemDao.Properties.Id.eq(id))
                .unique()
        val commentItem = commentListItem.commentItem
        commentListItem.delete()
        commentItem.delete()
    }

    override fun clear() {
        dataController.session.runInTx {
            val builder = dataController.session.commentItemDao.queryBuilder()
            val list = builder.build().listLazy()
            for (i in 0 until list.size) {
                list[i].likeStatus = CommentItem.LIKE_STATUS_NORMAL
            }
            updateCommentItems(list)
        }
    }

    override fun deleteComment(commentId: String, commentUrl: String, reportType: Int) {
        val comment = getCommentItemById(commentId)
        comment?.let {
            comment.status = CommentItem.STATUS_DELETED
            updateCommentItems(listOf(comment))
        }
    }
}