package com.under9.android.comments.ui.renderer

import android.graphics.drawable.ColorDrawable
import android.graphics.drawable.TransitionDrawable
import android.net.Uri
import android.os.Bundle
import android.text.Spannable
import android.text.SpannableStringBuilder
import android.text.style.ForegroundColorSpan
import android.text.style.StyleSpan
import android.util.ArrayMap
import android.view.View
import android.widget.FrameLayout
import android.widget.TextView
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.RecyclerView
import com.under9.android.comments.adapter.CommentItemClickListenerV2
import com.under9.android.comments.model.constant.CommentConstant
import com.under9.android.comments.model.ui.CommentItemThemeAttr
import com.under9.android.comments.model.wrapper.CommentItemWrapperInterface
import com.under9.android.comments.ui.renderer.ColorfulCommentModel.Companion.DEFAULT_BOARD_COLOR
import com.under9.android.comments.ui.renderer.CommentItemRendererDelegateV2.Companion.AVATAR_MODE_IMAGE
import com.under9.android.comments.ui.renderer.CommentItemRendererDelegateV2.Companion.KEY_AVATAR_MODE
import com.under9.android.comments.ui.renderer.CommentItemRendererDelegateV2.Companion.KEY_HIGHLIGHT_COMMENT_ID
import com.under9.android.comments.ui.renderer.CommentItemRendererDelegateV2.Companion.KEY_INDENTATION_LEVEL
import com.under9.android.comments.ui.renderer.CommentItemRendererDelegateV2.Companion.KEY_SHOULD_SHOW_COUNTRY
import com.under9.android.comments.ui.renderer.CommentItemRendererDelegateV2.Companion.KEY_SHOULD_SHOW_LOAD_NEXT_CONTAINER
import com.under9.android.comments.ui.renderer.CommentItemRendererDelegateV2.Companion.KEY_SHOULD_SHOW_LOAD_PREV_CONTAINER
import com.under9.android.comments.ui.renderer.CommentItemRendererDelegateV2.Companion.KEY_SHOULD_SHOW_OP
import com.under9.android.comments.ui.renderer.CommentItemRendererDelegateV2.Companion.KEY_VISIBLE_COMMENT_ONLINE_STATUS
import com.under9.android.comments.ui.widget.BaseCommentItemView
import com.under9.android.comments.ui.widget.BubbleCommentView
import com.under9.android.comments.ui.widget.CommentItemView
import com.under9.android.commentsystem.R
import com.under9.android.lib.network.UrlMapperInterface
import com.under9.android.lib.util.L10nUtil
import com.under9.android.lib.util.UiUtil
import com.under9.android.lib.widget.uiv.v3.UniversalImageView
import com.under9.android.lib.widget.uiv.v3.adapter.ImageTileInfoAdapter
import com.under9.android.lib.widget.uiv.v3.adapter.UIVAdapter
import com.under9.android.lib.widget.uiv.v3.adapter.VideoInfoAdapter
import com.under9.android.theme.ThemeStoreProvider
import kotlinx.android.synthetic.main.comment_system_list_item_inner_v5.view.*
import kotlinx.android.synthetic.main.comment_system_list_item_inner_v5.view.bottomPlaceholder
import kotlinx.android.synthetic.main.cs_list_item_bubble_view.view.*
import kotlin.math.pow

interface CommentItemInfoRendererInterface {
    fun bind(level: Int,
             wrapper: CommentItemWrapperInterface,
             themeAttr: CommentItemThemeAttr,
             viewHolder: RecyclerView.ViewHolder,
             position: Int)

    fun refreshRenderOptions(renderOptions: Bundle?)
}

abstract class BaseCommentItemRenderer(
        internal val commentItemClickListener: CommentItemClickListenerV2?,
        var renderOptions: Bundle?
) : CommentItemInfoRendererInterface {

    override fun bind(level: Int, wrapper: CommentItemWrapperInterface, themeAttr: CommentItemThemeAttr, viewHolder: RecyclerView.ViewHolder, position: Int) {
        bindCommentItemView(level, wrapper, themeAttr, viewHolder, viewHolder.itemView as BaseCommentItemView, position)
    }

    internal fun bindTagAndListener(commentItemWrapper: CommentItemWrapperInterface,
                                    view: View?, viewHolder: RecyclerView.ViewHolder, position: Int) {
        view?.apply {
            tag = commentItemWrapper
            setTag(R.id.commentPosition, position)
            setTag(R.id.commentItemViewHolder, viewHolder)
            setOnClickListener(commentItemClickListener)
        }
    }

    /**
     * After refreshing the list, call notifyDataSetChanged() to update the list rendering
     */
    override fun refreshRenderOptions(renderOptions: Bundle?) {
        this.renderOptions = renderOptions
    }

    abstract fun bindCommentItemView(level: Int,
                                     wrapper: CommentItemWrapperInterface,
                                     themeAttr: CommentItemThemeAttr,
                                     viewHolder: RecyclerView.ViewHolder, commentItemView: BaseCommentItemView, position: Int)

    fun shouldApplyMessageBackground(colorInt: Int, wrapper: CommentItemWrapperInterface) = colorInt != -1 && wrapper.getContent()?.isNotBlank() ?: run { false }
}


class AvatarRenderer(
        commentItemClickListener: CommentItemClickListenerV2,
        renderOption: Bundle?
) : BaseCommentItemRenderer(commentItemClickListener, renderOption) {

    private var dp16: Int = 0
    private var rootAvatarSize: Int = 0
    private var subLevelAvatarSize: Int = 0
    private var indentationLevel = 0
    private var avatarMode = 0
    private var visibleOnlineStatus = true

    init {
        refreshRenderOptions(renderOption)
    }

    override fun refreshRenderOptions(renderOptions: Bundle?) {
        super.refreshRenderOptions(renderOptions)
        indentationLevel = renderOptions?.getInt(KEY_INDENTATION_LEVEL, 0) ?: 0
        avatarMode = renderOptions?.getInt(KEY_AVATAR_MODE, AVATAR_MODE_IMAGE) ?: AVATAR_MODE_IMAGE
        visibleOnlineStatus = renderOptions?.getBoolean(KEY_VISIBLE_COMMENT_ONLINE_STATUS, true)
                ?: true
    }

    override fun bindCommentItemView(level: Int, wrapper: CommentItemWrapperInterface, themeAttr: CommentItemThemeAttr, viewHolder: RecyclerView.ViewHolder, commentItemView: BaseCommentItemView, position: Int) {
        var avatarUrl: String?
        with(commentItemView) {
            if (rootAvatarSize <= 0) {
                rootAvatarSize = resources.getDimensionPixelSize(R.dimen.avatar_size)
            }

            if (dp16 <= 0) {
                dp16 = UiUtil.dpToPx(context, 16)
            }

            val wrapperLevel = wrapper.getLevel() + indentationLevel - 1
            val avatarSize = (rootAvatarSize * 0.6.pow((wrapperLevel).toDouble())).toInt()
            val marginLeft = dp16 * (wrapperLevel)

            val layoutParams = avatar.layoutParams as ConstraintLayout.LayoutParams
            layoutParams.setMargins(marginLeft, layoutParams.topMargin, layoutParams.rightMargin, layoutParams.bottomMargin)
            layoutParams.width = avatarSize
            layoutParams.height = avatarSize

            val avatarLayoutParams = avatar.avatar.layoutParams as FrameLayout.LayoutParams
            avatarLayoutParams.width = avatarSize
            avatarLayoutParams.height = avatarSize

            val username = wrapper.getUser().getName()
            avatar.setTag(R.id.username, username)
            avatarUrl = wrapper.getUser().getAvatarUrl()

            when (avatarMode) {
                AVATAR_MODE_IMAGE -> avatar.setImageURI(if (avatarUrl == null) null else Uri.parse(avatarUrl))
                else -> setRoundAvatarColorDrawable(themeAttr.colorArray, themeAttr.textDrawableCache, themeAttr.avatarDrawable, username)
            }

            // if the own user(not commented users)  hidden the online status, they cannot see other online status, even the comment
            // user online
            if (visibleOnlineStatus) {
                // if own user(not commented users) set to show online indicator
                // And the comment is him/herself, then show the online status by default
                if (wrapper.isMyComment()) {
                    avatar.isActive = !wrapper.getUser().isHideActiveTs() // handle for show (hideActiveTs = 0)/ ninja(hideActiveTs = 1)
                } else {
                    avatar.isActive = wrapper.getUser().isActive() && !wrapper.getUser().isHideActiveTs() // isActive() is the user is actually active, but user can hide the online status (isHideActiveTs)
                }
            } else {
                // if the own user(not commented users) hidden the online indicator, default all comment user are hidden
                avatar.isActive = false
            }

            bindTagAndListener(wrapper, avatar, viewHolder, position)
        }
    }
}

open class UserInfoRenderer(
        commentItemClickListener: CommentItemClickListenerV2,
        renderOption: Bundle?,
        internal val userAccentColorMap: ArrayMap<String, Int>
) : BaseCommentItemRenderer(commentItemClickListener, renderOption) {

    private var shouldShowCountry: Boolean = false

    init {
        refreshRenderOptions(renderOption)
    }

    final override fun refreshRenderOptions(renderOptions: Bundle?) {
        super.refreshRenderOptions(renderOptions)
        shouldShowCountry = renderOptions?.getBoolean(KEY_SHOULD_SHOW_COUNTRY, false) ?: false
    }

    override fun bindCommentItemView(level: Int, wrapper: CommentItemWrapperInterface, themeAttr: CommentItemThemeAttr, viewHolder: RecyclerView.ViewHolder, commentItemView: BaseCommentItemView, position: Int) {
        val user = wrapper.getUser()
        val username = user.getName()
        val emojiStatus = user.getEmojiStatus()
        val country = if (shouldShowCountry) user.getCountryEmoji() else ""

        with(commentItemView) {
            userName.text = if (emojiStatus.isEmpty() && country.isEmpty()) {
                username
            } else if (emojiStatus.isEmpty() && country.isNotEmpty()) {
                "$username $country"
            } else if (emojiStatus.isNotEmpty() && country.isEmpty()) {
                "$username $emojiStatus"
            } else {
                "$username $country $emojiStatus"
            }
            userName.setTag(R.id.username, username)
            // apply the pro badge if that user is pro user
            if (user.isPro() && !user.isHideProBadge()) {
                val isProPlus = user.isProPlus()
                proBadge.updateProBadge(isProPlus = isProPlus)
                proBadge.visibility = View.VISIBLE
            } else {
                proBadge.visibility = View.GONE
            }

            val colorUsername = userAccentColorMap[wrapper.getUser().getAccentColor()]?.let { it } ?: run { DEFAULT_BOARD_COLOR }
            if (colorUsername != -1) {
                commentItemView.userName.setTextColor(colorUsername)
            }

            bindTagAndListener(wrapper, userName, viewHolder, position)
            bindTagAndListener(wrapper, proBadge, viewHolder, position)
        }
    }
}

class BubbleUserInfoRenderer(commentItemClickListener: CommentItemClickListenerV2,
                             renderOption: Bundle?,
                             userAccentColorMap: ArrayMap<String, Int>) : UserInfoRenderer(commentItemClickListener, renderOption, userAccentColorMap) {

    override fun bindCommentItemView(level: Int, wrapper: CommentItemWrapperInterface, themeAttr: CommentItemThemeAttr, viewHolder: RecyclerView.ViewHolder, commentItemView: BaseCommentItemView, position: Int) {
        val bubbleBackgroundInt = userAccentColorMap[wrapper.getUser().getAccentColor()]?.let { it } ?: run { DEFAULT_BOARD_COLOR }
        if (shouldApplyMessageBackground(bubbleBackgroundInt, wrapper)) {
            if (bubbleBackgroundInt != -1) {
                commentItemView.proBadge.fixedBadgeTextColor = bubbleBackgroundInt
            }
        } else {
            commentItemView.proBadge.resetBadgeTextColor()
        }
        super.bindCommentItemView(level, wrapper, themeAttr, viewHolder, commentItemView, position)
    }
}

open class OpInfoRenderer(
        commentItemClickListener: CommentItemClickListenerV2,
        renderOption: Bundle?
) : BaseCommentItemRenderer(commentItemClickListener, renderOption) {
    private var showOP = true
    private val PREFIX_OP = "(OP)"
    init {
        refreshRenderOptions(renderOption)
    }

    final override fun refreshRenderOptions(renderOptions: Bundle?) {
        super.refreshRenderOptions(renderOptions)
        showOP = renderOptions?.getBoolean(KEY_SHOULD_SHOW_OP) ?: true
    }

    override fun bindCommentItemView(level: Int, wrapper: CommentItemWrapperInterface, themeAttr: CommentItemThemeAttr, viewHolder: RecyclerView.ViewHolder, commentItemView: BaseCommentItemView, position: Int) {
        with(commentItemView) {
            if (showOP && wrapper.isOP) {
                val opName = "$PREFIX_OP ${userName.text}"
                userName.text = opName
            } else {
                userName.text = userName.text
            }

            // if user don't set username color, use the default one
            if (wrapper.getUser().getAccentColor().isBlank()) {
                userName.setTextColor(themeAttr.getColor(R.attr.under9_themeTextColorPrimary))
            }
        }
    }
}

class BubbleOpInfoRenderer(commentItemClickListener: CommentItemClickListenerV2,
                           renderOption: Bundle?) : OpInfoRenderer(commentItemClickListener, renderOption) {

    override fun bindCommentItemView(level: Int, wrapper: CommentItemWrapperInterface, themeAttr: CommentItemThemeAttr, viewHolder: RecyclerView.ViewHolder, commentItemView: BaseCommentItemView, position: Int) {
        super.bindCommentItemView(level, wrapper, themeAttr, viewHolder, commentItemView, position)
        // case if Text only, then should have bubble color background, and change the username color to white
        if (wrapper.getUser().getAccentColor().isNotBlank() && wrapper.getType() == CommentConstant.TYPE_TEXT) {
            commentItemView.userName.setTextColor(ContextCompat.getColor(viewHolder.itemView.context, R.color.under9_theme_white))
        } else if (wrapper.getUser().getAccentColor().isNotBlank() && wrapper.getType() == CommentConstant.TYPE_MEDIA && wrapper.getContent()?.isNotBlank() ?: run { false }) {
            // case if user write a comment with media AND text, imply should have bubble color background, and change the username color to white
            commentItemView.userName.setTextColor(ContextCompat.getColor(viewHolder.itemView.context, R.color.under9_theme_white))
        } else {
            // case if just a media, change to default username color 
            commentItemView.userName.setTextColor(themeAttr.getColor(R.attr.under9_themeTextColorPrimary))
        }
    }
}

open class TextContentRenderer(
        commentItemClickListener: CommentItemClickListenerV2,
        renderOption: Bundle?
) : BaseCommentItemRenderer(commentItemClickListener, renderOption) {

    private var highlightCommentId: String? = null
    private var shouldUsernameClickable: Boolean = false
    private var shouldHighlightUrl: Boolean = false

    init {
        refreshRenderOptions(renderOption)
    }

    final override fun refreshRenderOptions(renderOptions: Bundle?) {
        super.refreshRenderOptions(renderOptions)
        highlightCommentId = renderOptions?.getString(KEY_HIGHLIGHT_COMMENT_ID, null)
        shouldUsernameClickable = renderOptions?.getBoolean(CommentItemRendererDelegateV2.KEY_SHOULD_USERNAME_CLICKABLE, false)
                ?: false
        shouldHighlightUrl = renderOptions?.getBoolean(CommentItemRendererDelegateV2.KEY_SHOULD_HIGHLIGHT_URL, false)
                ?: false
    }

    override fun bindCommentItemView(level: Int, wrapper: CommentItemWrapperInterface, themeAttr: CommentItemThemeAttr, viewHolder: RecyclerView.ViewHolder, commentItemView: BaseCommentItemView, position: Int) {
        val textContent = wrapper.getContent()
        val content = commentItemView.content

        if (textContent.isNullOrEmpty()) {
            content.visibility = View.GONE
        } else {
            content.visibility = View.VISIBLE
        }

        val contentBuilder = SpannableStringBuilder(textContent)
        if (shouldUsernameClickable) {
            val matcher = CommentItemThemeAttr.mentionedUserPattern.matcher(textContent)
            while (matcher.find()) {
                val start = matcher.start()
                val end = matcher.end()
                val username = matcher.group()
                contentBuilder.setSpan(getMentionStyleSpan(themeAttr), start, end, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
                contentBuilder.setSpan(MentionedUserClickSpan(username, wrapper.getMentionedAccountId(username), commentItemClickListener), start, end, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
            }
        }

        if (shouldHighlightUrl) {
            val matcher = CommentItemThemeAttr.urlPatternV2.matcher(textContent)
            while (matcher.find()) {
                val start = matcher.start()
                val end = matcher.end()
                val url = matcher.group()
                contentBuilder.setSpan(getMentionStyleSpan(themeAttr), start, end, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
                contentBuilder.setSpan(UrlClickSpan(wrapper, url, commentItemClickListener), start, end, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
            }
        }

        if (wrapper.shouldHighlight() || wrapper.getCommentId() == highlightCommentId) {
            content.setTextColor(themeAttr.getColor(R.attr.under9_themeTextColorPrimary))
        } else {
            content.setTextColor(themeAttr.getColor(R.attr.under9_themeTextColorPrimary))
        }

        content.text = contentBuilder
//        content.movementMethod = BetterLinkMovementMethod.getInstance()
//        content.isDuplicateParentStateEnabled = true

//        bindTagAndListener(wrapper, content, viewHolder, position)
    }

    open fun getMentionStyleSpan(themeAttr: CommentItemThemeAttr): Any {
        return ForegroundColorSpan(themeAttr.getColor(R.attr.under9_themeColorAccent))
    }
}

class BubbleTextContentRenderer(commentItemClickListener: CommentItemClickListenerV2,
                                renderOption: Bundle?) : TextContentRenderer(commentItemClickListener, renderOption) {
    override fun bindCommentItemView(level: Int, wrapper: CommentItemWrapperInterface, themeAttr: CommentItemThemeAttr, viewHolder: RecyclerView.ViewHolder, commentItemView: BaseCommentItemView, position: Int) {
        super.bindCommentItemView(level, wrapper, themeAttr, viewHolder, commentItemView, position)

        if (wrapper.getUser().getAccentColor().isNotBlank()) {
            val content = commentItemView.content
            content.setTextColor(ContextCompat.getColor(viewHolder.itemView.context, R.color.under9_theme_white))
        }
    }

    override fun getMentionStyleSpan(themeAttr: CommentItemThemeAttr) = StyleSpan(android.graphics.Typeface.BOLD)
}

open class BodyRenderer(
        commentItemClickListener: CommentItemClickListenerV2,
        renderOption: Bundle?
) : BaseCommentItemRenderer(commentItemClickListener, renderOption) {

    private var highlightCommentId: String? = null
    private var showLoadPrevContainer = false
    private var showLoadNextContainer = true

    // Experiment
    internal var clickableBody = false

    private var highlightedColor = -1
    private var foregroundColor = -1

    init {
        refreshRenderOptions(renderOption)
    }

    final override fun refreshRenderOptions(renderOptions: Bundle?) {
        super.refreshRenderOptions(renderOptions)
        renderOptions?.run {
            highlightCommentId = getString(KEY_HIGHLIGHT_COMMENT_ID, null)
            showLoadPrevContainer = getBoolean(KEY_SHOULD_SHOW_LOAD_PREV_CONTAINER, false)
            showLoadNextContainer = getBoolean(KEY_SHOULD_SHOW_LOAD_NEXT_CONTAINER, true)
        }
    }

    override fun bindCommentItemView(level: Int, wrapper: CommentItemWrapperInterface, themeAttr: CommentItemThemeAttr, viewHolder: RecyclerView.ViewHolder, baseCommentItemView: BaseCommentItemView, position: Int) {
        with(baseCommentItemView) {
            baseCommentItemView.progressBar.visibility = View.GONE

            if (highlightedColor == -1) {
                highlightedColor = UiUtil.themeAttributeToColor(R.attr.cs_themeForegroundHighlighted, context, -1)
            }

            if (foregroundColor == -1) {
                foregroundColor = UiUtil.themeAttributeToColor(R.attr.under9_themeForeground, context, -1)
            }

            if (showLoadNextContainer && level <= 1 && wrapper.getChildrenTotal() > 0) {
                baseCommentItemView.loadMoreTxt.visibility = View.VISIBLE
                baseCommentItemView.loadMoreContainer.visibility = View.VISIBLE
                baseCommentItemView.loadMoreTxt.text = L10nUtil.getFormattedQuantityString(context, R.plurals.view_n_replies, wrapper.getChildrenTotal())
                baseCommentItemView.bottomPlaceholder.visibility = View.GONE
            } else {
                baseCommentItemView.loadMoreTxt.visibility = View.GONE
                baseCommentItemView.loadMoreContainer.visibility = View.GONE
                baseCommentItemView.bottomPlaceholder.visibility = View.VISIBLE
            }

            if (showLoadPrevContainer && wrapper.isParent() && wrapper.prevUrl != null && position == 0) {
                baseCommentItemView.loadPrevTxt.visibility = View.VISIBLE
                baseCommentItemView.loadPrevContainer.visibility = View.VISIBLE
            } else {
                baseCommentItemView.loadPrevTxt.visibility = View.GONE
                baseCommentItemView.loadPrevContainer.visibility = View.GONE
            }

            this.setBackgroundColor(foregroundColor)
            if (baseCommentItemView is CommentItemView) {
                commentBorder.setBackgroundColor(themeAttr.getColor(R.attr.under9_themeLineColor))
            }

            when {
                wrapper.getCommentId() == highlightCommentId -> baseCommentItemView.setBackgroundColor(UiUtil.themeAttributeToColor(R.attr.cs_themeForegroundHighlighted, context, -1))
                wrapper.shouldHighlight() -> {
                    val colorDrawbles = arrayOf(ColorDrawable(highlightedColor), ColorDrawable(foregroundColor))
                    val transitionDrawable = TransitionDrawable(colorDrawbles)
                    transitionDrawable.isCrossFadeEnabled = true
                    baseCommentItemView.background = transitionDrawable
                    transitionDrawable.startTransition(5000)
                }
                else -> {
                    baseCommentItemView.setBackgroundColor(foregroundColor)
                    meta.setTextColor(themeAttr.getColor(R.attr.under9_themeTextColorSecondary))
                }
            }

            if (wrapper.isPending()) {
                meta.text = "..."
            } else {
                meta.text = themeAttr.timeFormatter.formatTime(wrapper.getTime())
            }

            bindTagAndListener(wrapper, root, viewHolder, position)
            bindTagAndListener(wrapper, moreBtn, viewHolder, position)
            bindTagAndListener(wrapper, loadMoreContainer, viewHolder, position)
            bindTagAndListener(wrapper, loadPrevContainer, viewHolder, position)
            bindTagAndListener(wrapper, replyBtn, viewHolder, position)
            viewHolder.itemView.tag = wrapper
        }
    }

}


class BubbleBodyRenderer(
        commentItemClickListener: CommentItemClickListenerV2,
        renderOption: Bundle?,
        private val userAccentColorMap: ArrayMap<String, Int>
) : BodyRenderer(commentItemClickListener, renderOption) {

    init {
        clickableBody = true
    }
    override fun bindCommentItemView(level: Int, wrapper: CommentItemWrapperInterface, themeAttr: CommentItemThemeAttr, viewHolder: RecyclerView.ViewHolder, baseCommentItemView: BaseCommentItemView, position: Int) {
        super.bindCommentItemView(level, wrapper, themeAttr, viewHolder, baseCommentItemView, position)
        with(baseCommentItemView) {
            if (baseCommentItemView is BubbleCommentView) {
                bindTagAndListener(wrapper, textBubbleBackground, viewHolder, position)

                // if the color is not exist in the map, use the default color
                val bubbleBackgroundInt = userAccentColorMap[wrapper.getUser().getAccentColor()]?.let { it } ?: run { DEFAULT_BOARD_COLOR }

                when {
                    // if bubble background applied, meta should set to white
                    shouldApplyMessageBackground(bubbleBackgroundInt, wrapper) -> {
                        meta.setTextColor(ContextCompat.getColor(viewHolder.itemView.context, R.color.under9_theme_white60))
                        baseCommentItemView.textBubbleBackground.setCardBackgroundColor(bubbleBackgroundInt)
                    }

                    // if the user used default bubble color, apply the default bubble color
                    wrapper.getContent().isNullOrBlank() -> {
                        textBubbleBackground.setCardBackgroundColor(UiUtil.themeAttributeToColor(R.attr.under9_themeForeground, context, -1))
                    }
                    else -> {
                        textBubbleBackground.setCardBackgroundColor(UiUtil.themeAttributeToColor(R.attr.cs_boardBubbleBackground, context, -1))
                    }
                }
                bindTagAndListener(wrapper, textBubbleBackground, viewHolder, position)

            }
        }
    }
}

class BubbleVoteInfoRenderer(commentItemClickListener: CommentItemClickListenerV2?,
                             renderOption: Bundle?): VoteInfoRenderer(commentItemClickListener, renderOption) {

    override fun bindCommentItemView(level: Int, wrapper: CommentItemWrapperInterface, themeAttr: CommentItemThemeAttr, viewHolder: RecyclerView.ViewHolder, baseCommentItemView: BaseCommentItemView, position: Int) {
        super.bindCommentItemView(level, wrapper, themeAttr, viewHolder, baseCommentItemView, position)
        bindTagAndListener(wrapper, baseCommentItemView.circleLikeBtn, viewHolder, position)
        bindTagAndListener(wrapper, baseCommentItemView.circleLikeContainer, viewHolder, position)
    }

    override fun pureBind(wrapper: CommentItemWrapperInterface, baseCommentItemView: BaseCommentItemView) {
        super.pureBind(wrapper, baseCommentItemView)
        with(baseCommentItemView) {

            if (accentColor == -1) {
                accentColor = com.under9.android.lib.util.UiUtil.themeAttributeToColor(com.under9.android.commentsystem.R.attr.under9_themeColorAccent, context, -1)
            }

            if (normalColor == -1) {
                normalColor = com.under9.android.lib.util.UiUtil.themeAttributeToColor(com.under9.android.commentsystem.R.attr.under9_themeTextColorSecondary, context, -1)
            }

            when (wrapper.likeStatus) {
                com.under9.android.comments.model.CommentItem.LIKE_STATUS_LIKED -> {
                    circleLikeBtn.isChecked = true
                    circleLikeBtn.setTextColor(accentColor)
                }

                com.under9.android.comments.model.CommentItem.LIKE_STATUS_NORMAL,
                com.under9.android.comments.model.CommentItem.LIKE_STATUS_DISLIKED  -> {
                    circleLikeBtn.isChecked = false
                    circleLikeBtn.setTextColor(normalColor)
                }
            }

            circleLikeBtn.buttonDrawable = androidx.appcompat.content.res.AppCompatResources.getDrawable(context, com.under9.android.commentsystem.R.drawable.ic_comment_upvote_v2)
            if (context is ThemeStoreProvider) {
                circleLikeBtn.buttonDrawable = com.under9.android.lib.util.ColorUtils.getTintedVectorDrawable(context,
                        com.under9.android.commentsystem.R.drawable.ic_comment_upvote_v2,
                        (context as ThemeStoreProvider).getThemeStore().checkedColorStateList)
            }
        }
    }

}
open class VoteInfoRenderer(
        commentItemClickListener: CommentItemClickListenerV2?,
        renderOption: Bundle?
) : BaseCommentItemRenderer(commentItemClickListener, renderOption) {

    protected var accentColor = -1
    protected var normalColor = -1

    override fun bindCommentItemView(level: Int, wrapper: CommentItemWrapperInterface, themeAttr: CommentItemThemeAttr, viewHolder: RecyclerView.ViewHolder, baseCommentItemView: BaseCommentItemView, position: Int) {
        pureBind(wrapper, baseCommentItemView)

        bindTagAndListener(wrapper, baseCommentItemView.likeBtn, viewHolder, position)
        bindTagAndListener(wrapper, baseCommentItemView.dislikeBtn, viewHolder, position)
    }

    open fun pureBind(wrapper: CommentItemWrapperInterface, baseCommentItemView: BaseCommentItemView) {
        with(baseCommentItemView) {

            if (accentColor == -1) {
                accentColor = com.under9.android.lib.util.UiUtil.themeAttributeToColor(com.under9.android.commentsystem.R.attr.under9_themeColorAccent, context, -1)
            }

            if (normalColor == -1) {
                normalColor = com.under9.android.lib.util.UiUtil.themeAttributeToColor(com.under9.android.commentsystem.R.attr.under9_themeTextColorSecondary, context, -1)
            }

            when (wrapper.likeStatus) {
                com.under9.android.comments.model.CommentItem.LIKE_STATUS_LIKED -> {
                    likeBtn.isChecked = true
                    dislikeBtn.isChecked = false

                    likeBtn.setTextColor(accentColor)
                    likeBtnMask.setTextColor(accentColor)
                    dislikeBtn.setTextColor(normalColor)
                    dislikeBtnMask.setTextColor(normalColor)
                }

                com.under9.android.comments.model.CommentItem.LIKE_STATUS_NORMAL -> {
                    likeBtn.isChecked = false
                    dislikeBtn.isChecked = false

                    likeBtn.setTextColor(normalColor)
                    likeBtnMask.setTextColor(normalColor)
                    dislikeBtn.setTextColor(normalColor)
                    dislikeBtnMask.setTextColor(normalColor)
                }

                com.under9.android.comments.model.CommentItem.LIKE_STATUS_DISLIKED -> {
                    likeBtn.isChecked = false
                    dislikeBtn.isChecked = true

                    likeBtn.setTextColor(normalColor)
                    likeBtnMask.setTextColor(normalColor)
                    dislikeBtn.setTextColor(accentColor)
                    dislikeBtnMask.setTextColor(accentColor)
                }
            }


            setVoteCount(wrapper.likeCount, likeBtn)
            setVoteCount(wrapper.dislikeCount, dislikeBtn)

            likeBtn.buttonDrawable = androidx.appcompat.content.res.AppCompatResources.getDrawable(context, com.under9.android.commentsystem.R.drawable.ic_comment_upvote_v2)
            dislikeBtn.buttonDrawable = androidx.appcompat.content.res.AppCompatResources.getDrawable(context, com.under9.android.commentsystem.R.drawable.ic_comment_downvote_v2)
            if (context is ThemeStoreProvider) {
                likeBtn.buttonDrawable = com.under9.android.lib.util.ColorUtils.getTintedVectorDrawable(context,
                        com.under9.android.commentsystem.R.drawable.ic_comment_upvote_v2,
                        (context as ThemeStoreProvider).getThemeStore().checkedColorStateList)

                dislikeBtn.buttonDrawable = com.under9.android.lib.util.ColorUtils.getTintedVectorDrawable(context,
                        com.under9.android.commentsystem.R.drawable.ic_comment_downvote_v2,
                        (context as ThemeStoreProvider).getThemeStore().checkedColorStateList)
            }
        }
    }

    private fun setVoteCount(count: Int, textView: TextView) {
        if (count == 0) {
            textView.text = ""
        } else {
            textView.text = L10nUtil.getFormatNumber(count)
        }
    }

}

open class MediaRenderer(
        commentItemClickListener: CommentItemClickListenerV2,
        renderOption: Bundle?,
        private val urlMapperInterface: UrlMapperInterface
) : BaseCommentItemRenderer(commentItemClickListener, renderOption) {

    override fun bindCommentItemView(level: Int, wrapper: CommentItemWrapperInterface, themeAttr: CommentItemThemeAttr, viewHolder: RecyclerView.ViewHolder, baseCommentItemView: BaseCommentItemView, position: Int) {
        val existingWrapper = baseCommentItemView.uiv.getTag(R.id.comment_wrapper)

        if (wrapper.getType() != CommentConstant.TYPE_MEDIA) {
            baseCommentItemView.uiv.visibility = View.GONE
            return
        } else {
            baseCommentItemView.uiv.visibility = View.VISIBLE
        }

        if (existingWrapper != null && existingWrapper == wrapper) return

        val imageMediaByType = wrapper.getFirstMedia()?.imageMetaByType

        when (imageMediaByType?.type ?: return) {
            CommentConstant.MEDIA_TYPE_STATIC -> {
                val embedImage = imageMediaByType.image
                val height = embedImage.height
                val width = embedImage.width
                val url = embedImage.getImageUrl()

                val imageTileInfoAdapter = ImageTileInfoAdapter.newBuilder()
                        .setImage(url, width, height)
                        .build()

                val adapter = UIVAdapter.newBuilder(UniversalImageView.MODE_STATIC_IMAGE)
                        .setUrlMapper(urlMapperInterface)
                        .setRetryDrawable(R.drawable.ic_media_reload)
                        .setImageTileInfoAdapter(imageTileInfoAdapter)
                        .setOnClickListener(commentItemClickListener)
                        .setOnLongClickListener(commentItemClickListener)
                        .build()
                baseCommentItemView.uiv.adapter = adapter
            }

            CommentConstant.MEDIA_TYPE_ANIMATED -> {
                val video = imageMediaByType.video
                val image = imageMediaByType.image

                if (video != null && image != null) {
                    val url = video.url

                    val height = image.height

                    val videoInfoAdapter = VideoInfoAdapter.newBuilder()
                            .videoUrl(url)
                            .build()

                    val imageTileInfoAdapter = ImageTileInfoAdapter.newBuilder()
                            .setImage(image.getImageUrl(), image.width, height)
                            .build()

                    val adapter = UIVAdapter.newBuilder(UniversalImageView.MODE_MP4_WITHOUT_SOUND)
                            .setRetryDrawable(R.drawable.ic_media_reload)
                            .centerBadge(R.drawable.ic_play_gif)
                            .setUrlMapper(urlMapperInterface)
                            .setImageTileInfoAdapter(imageTileInfoAdapter)
                            .setVideoInfoAdapter(videoInfoAdapter)
                            .setOnLongClickListener(commentItemClickListener)
                            .build()

                    baseCommentItemView.uiv.adapter = adapter
                } else {
                    return
                }
            }
        }

        bindTagAndListener(wrapper, baseCommentItemView.refresh, viewHolder, position)
        baseCommentItemView.uiv.apply {
            setTag(R.id.commentPosition, position)
            setTag(R.id.comment_wrapper, wrapper)
            setTag(R.id.image_holder, viewHolder)
        }
    }
}

class BubbleMediaRenderer(
        commentItemClickListener: CommentItemClickListenerV2,
        renderOption: Bundle?,
        urlMapperInterface: UrlMapperInterface) : MediaRenderer(commentItemClickListener, renderOption, urlMapperInterface) {

    override fun bindCommentItemView(level: Int, wrapper: CommentItemWrapperInterface, themeAttr: CommentItemThemeAttr, viewHolder: RecyclerView.ViewHolder, baseCommentItemView: BaseCommentItemView, position: Int) {
        super.bindCommentItemView(level, wrapper, themeAttr, viewHolder, baseCommentItemView, position)
        if (wrapper.getType() != CommentConstant.TYPE_MEDIA) {
            (baseCommentItemView as BubbleCommentView).uivBubbleContainer.visibility = View.GONE
        } else {
            (baseCommentItemView as BubbleCommentView).uivBubbleContainer.visibility = View.VISIBLE
        }
    }
}