package com.under9.android.comments.ui.widget

import android.content.Context
import android.os.Build
import android.util.AttributeSet
import android.util.TypedValue
import android.view.LayoutInflater
import android.view.View
import android.widget.*
import androidx.annotation.AttrRes
import androidx.annotation.DrawableRes
import androidx.annotation.StyleRes
import androidx.appcompat.content.res.AppCompatResources
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.core.content.ContextCompat
import androidx.dynamicanimation.animation.SpringAnimation
import androidx.dynamicanimation.animation.SpringForce
import com.under9.android.comments.model.ui.CommentItemThemeAttr
import com.under9.android.commentsystem.R
import com.under9.android.lib.util.L10nUtil
import com.under9.android.lib.util.StringUtils
import com.under9.android.lib.util.UiUtil
import com.under9.android.lib.widget.ActiveAvatarView
import com.under9.android.lib.widget.ProBadgeView
import com.under9.android.lib.widget.text.TextDrawable
import com.under9.android.lib.widget.uiv.v3.UniversalImageView

abstract class BaseCommentItemView: ConstraintLayout {

    constructor(context: Context): this(context, null, -1)

    constructor(context: Context, attributeSet: AttributeSet?) : this(context, attributeSet, -1)

    constructor(context: Context, @AttrRes attributeSet: AttributeSet?, @StyleRes defStyleAttr: Int) : super(context, attributeSet, defStyleAttr) {
        attributeSet?.let {
            loadAttributes(context, attributeSet, defStyleAttr)
        }
    }
    protected var layoutId: Int = 0
    lateinit var commentItemThemeAttr: CommentItemThemeAttr
    lateinit var root: ConstraintLayout
    lateinit var avatar: ActiveAvatarView
    lateinit var userName: TextView
    lateinit var proBadge: ProBadgeView
    lateinit var meta: TextView
    lateinit var content: TextView
    lateinit var uiv: UniversalImageView
    lateinit var replyBtn: TextView
    lateinit var likeBtn: CheckBox
    lateinit var likeBtnMask: CheckBox
    lateinit var dislikeBtn: CheckBox
    lateinit var dislikeBtnMask: CheckBox
    lateinit var moreBtn: ImageButton
    lateinit var loadMoreContainer: View
    lateinit var loadMoreTxt: TextView
    lateinit var loadPrevTxt: TextView
    lateinit var loadPrevContainer: TextView
    lateinit var progressBar: ProgressBar
    lateinit var refresh: ImageView

    open fun initView() {
        LayoutInflater.from(context).inflate(layoutId, this, true)
        id = R.id.cs_rootView
        root = this

        if (Build.VERSION.SDK_INT >= 23) {
            val outValue = TypedValue()
            context.theme.resolveAttribute(android.R.attr.selectableItemBackground, outValue, true)
            foreground = ContextCompat.getDrawable(context, outValue.resourceId)
        }

        avatar = findViewById(R.id.avatar)
        userName = findViewById(R.id.userName)
        proBadge = findViewById(R.id.proBadge)
        meta = findViewById(R.id.meta)
        content = findViewById(R.id.content)
        uiv = findViewById(R.id.image)
        replyBtn = findViewById(R.id.replyBtnV4)
        likeBtn = findViewById(R.id.likeBtn)
        likeBtnMask = findViewById(R.id.likeBtnMask)
        likeBtnMask.isClickable = false
        likeBtnMask.visibility = View.GONE

        dislikeBtn = findViewById(R.id.dislikeBtn)
        dislikeBtnMask = findViewById(R.id.dislikeBtnMask)
        dislikeBtnMask.isClickable = false
        dislikeBtnMask.visibility = View.GONE

        moreBtn = findViewById(R.id.moreBtn)
        loadMoreTxt = findViewById(R.id.loadMoreTxt)
        loadMoreContainer = findViewById(R.id.loadMoreContainer)
        loadPrevTxt = findViewById(R.id.loadPrevTxt)
        loadPrevContainer = findViewById(R.id.loadPrevContainer)
        progressBar = findViewById(R.id.progress)
        refresh = findViewById(R.id.refresh)
    }

    protected fun loadAttributes(context: Context, attrs: AttributeSet, deftStyleAttr: Int) {
        val a = context.theme.obtainStyledAttributes(attrs,
                R.styleable.CommentItemView, deftStyleAttr, 0)
        val enableVoteMask = a.getBoolean(R.styleable.CommentItemView_enable_vote_mask, false)
        if (enableVoteMask) {
            likeBtnMask.visibility = View.VISIBLE
            dislikeBtnMask.visibility = View.VISIBLE
        } else {
            likeBtnMask.visibility = View.GONE
            dislikeBtnMask.visibility = View.GONE
        }
    }

    fun setAvatar(uri: String?) {
        this.avatar.setImageURI(uri)
    }

    fun setRoundAvatarColorDrawable(colors: IntArray,
                                    cache: Array<TextDrawable?>,
                                    @DrawableRes imageDrawable: Int,
                                    username: String) {
        val hashedColorIndex = StringUtils.getHashedSlot(username, colors.size)
        if (cache[hashedColorIndex] == null) {
            cache[hashedColorIndex] = TextDrawable.builder()
                    .buildRound("", colors[hashedColorIndex])
        }

        this.avatar.setImageBackground(cache[hashedColorIndex])
        this.avatar.setImageDrawable(AppCompatResources.getDrawable(context, imageDrawable))
    }

    fun setPointStatus(userPoint: Int) {
        var likeSelected = false
        var dislikeSelected = false

        if (userPoint == -1) {
            dislikeSelected = true
        } else if (userPoint == 1) {
            likeSelected = true
        }

        likeBtn.isChecked = likeSelected
        likeBtnMask.isChecked = likeSelected

        dislikeBtn.isChecked = dislikeSelected
        dislikeBtn.isChecked = dislikeSelected
    }

    fun setLikeStatus(likeCount: Int, dislikeCount: Int,
                      status: Int, shouldHighlight: Boolean, withAnimation: Boolean) {
        setVoteStatus(likeBtn, likeCount, status, shouldHighlight)
        setVoteStatus(dislikeBtn, dislikeCount, -status, shouldHighlight)

        likeBtn.isChecked = status == 1
        likeBtnMask.isChecked = likeBtn.isChecked

        dislikeBtn.isChecked = false
        dislikeBtnMask.isChecked = false

        if (withAnimation) {
            runAnimation(likeBtn, status)
        }
    }

    fun setDiskLikeStatus(likeCount: Int, dislikeCount: Int,
                          status: Int, shouldHighlight: Boolean, withAnimation: Boolean) {
        setVoteStatus(likeBtn, likeCount, status, shouldHighlight)
        setVoteStatus(dislikeBtn, dislikeCount, -status, shouldHighlight)

        likeBtn.isChecked = false
        likeBtnMask.isChecked = false

        dislikeBtn.isChecked = status == -1
        dislikeBtnMask.isChecked = dislikeBtn.isChecked

        if (withAnimation) {
            runAnimation(dislikeBtn, status)
        }
    }

    fun runAnimation(v: View, status: Int) {
        val startValue = when (status) {
            1 -> -10f
            -1 -> 10f
            else -> 0f
        }

        val animation = SpringAnimation(v, SpringAnimation.TRANSLATION_Y)
        val springForce = SpringForce()
        springForce.finalPosition = 0f
        springForce.stiffness = SpringForce.STIFFNESS_MEDIUM
        springForce.dampingRatio = SpringForce.DAMPING_RATIO_MEDIUM_BOUNCY
        animation.setStartValue(startValue)
        animation.spring = springForce
        animation.start()
    }

    protected fun setVoteStatus(countTextView: TextView,
                              count: Int, status: Int, shouldHighlight: Boolean) {
        if (!shouldHighlight) {
            val color = getLikeCountColorByStatus(status)
            val resolvedColor = UiUtil.themeAttributeToColor(color, context, -1)
            countTextView.setTextColor(resolvedColor)
        }

        if (count == 0) {
            countTextView.text = ""
        } else {
            countTextView.text = L10nUtil.getFormatNumber(count)
        }
    }

    @AttrRes
    protected fun getLikeCountColorByStatus(likeStatus: Int): Int {
        return when {
            likeStatus > 0 -> R.attr.under9_themeColorAccent
            likeStatus == 0 -> R.attr.under9_themeTextColorSecondary
            else -> R.attr.under9_themeTextColorSecondary
        }
    }
}