package com.under9.android.lib.blitz.delegate

import android.widget.LinearLayout
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.under9.android.lib.blitz.BlitzActionListener
import com.under9.android.lib.blitz.BlitzState
import com.under9.android.lib.blitz.BlitzView
import timber.log.Timber

/**
 * StrictLinearPaginationDetector is different from LinearPaginationDetector
 * Which can define the number of item to trigger load next/ prev
 * but LinearPaginationDetector is using @param loadMoreBound to trigger load next/prev, which is not
 * suitable for load next and prev support (e.g board)
 */

class StrictLinearPaginationDetector @JvmOverloads constructor(private val actionListener: BlitzActionListener,
                                                               private val preTriggerNextCount:Int,
                                                               private val preTriggerPrevCount: Int,
                                                               private val needStackFromEnd: Boolean = true) : PaginationDetector {

    private var orientation = VERTICAL
    private var shouldTriggeredOnAttachment = true

    private var lastTriggerLoadNext: Long = 0
    private var lastTriggerLoadPrev: Long = 0


    override fun checkPagination(blitzView: BlitzView, layoutManager: RecyclerView.LayoutManager?, dx: Int, dy: Int) {
        val linearLayoutManager = layoutManager as LinearLayoutManager?
        if (VERTICAL == orientation) {
            checkLoadVertical(blitzView, linearLayoutManager, dy)
        } else {
            checkLoadHorizontal(blitzView, linearLayoutManager, dx)
        }
    }

    /**
     * By default, [android.support.v7.widget.RecyclerView.OnScrollListener.onScrolled]
     * will be triggered after scroll listener is added, we use this to configure
     * if we want this behaviour
     */
    override fun shouldTriggerOnAttachment(): Boolean {
        return this.shouldTriggeredOnAttachment
    }

    private fun checkLoadVertical(blitzView: BlitzView, linearLayoutManager: LinearLayoutManager?, dy: Int) {

        val now = System.currentTimeMillis()
        val viewState = blitzView.viewState

        if (linearLayoutManager == null) return
        if (viewState == BlitzState.STATE_INIT_LOADING ||
                viewState == BlitzState.STATE_REFRESHING ||
                viewState == BlitzState.STATE_LOADING_MORE)
            return

        val itemCount = linearLayoutManager.itemCount
        val visibleItemCount = linearLayoutManager.childCount
        val firstVisibleItem = linearLayoutManager.findFirstVisibleItemPosition()

        Timber.d("checkLoadVertical: itemCount=" + itemCount +
                "\n visibleItemCount=" + visibleItemCount +
                "\nfirstVisibleItem=" + firstVisibleItem +
                "\ndy = $dy, " +
                "\nlastTriggerLoadNext=$lastTriggerLoadNext, diff=${now - lastTriggerLoadNext}, " +
                "\nlastTriggerLoadPrev=$lastTriggerLoadPrev, diff=${now - lastTriggerLoadPrev}, " +
                "\nviewState=$viewState, firstVisible=$firstVisibleItem, " +
                "\nprevTriggerPrev=$preTriggerPrevCount, prevTriggerNext=$preTriggerNextCount"
        )

        if (dy >= 0 && firstVisibleItem + visibleItemCount >= itemCount - preTriggerNextCount) {
            if (now - lastTriggerLoadNext < LOAD_MORE_CHECK_RATE_LIMIT) {
                return
            }
            if (viewState == BlitzState.STATE_FINISH_LOADING_NEXT_END_OF_LIST) return
            if (actionListener.onTriggerLoadNext() && linearLayoutManager.stackFromEnd && needStackFromEnd) {
                blitzView.recyclerView.post {
                    linearLayoutManager.stackFromEnd = false
                }
            }
            lastTriggerLoadNext = now

            Timber.d("checkLoadVertical: --loadNext=$lastTriggerLoadNext")
        }


        if (dy <= 0 && firstVisibleItem - preTriggerPrevCount <= 0) {
            if (now - lastTriggerLoadPrev < LOAD_MORE_CHECK_RATE_LIMIT) return
            if (viewState == BlitzState.STATE_FINISH_LOADING_PREV_END_OF_LIST) return
            if (actionListener.onTriggerLoadPrevious() && !linearLayoutManager.stackFromEnd && needStackFromEnd) {
                blitzView.recyclerView.post {
                    linearLayoutManager.stackFromEnd = true
                }
                lastTriggerLoadPrev = now
            }

            Timber.d("checkLoadVertical: --loadPrev=$lastTriggerLoadPrev")
        }
    }

    private fun checkLoadHorizontal(blitzView: BlitzView, linearLayoutManager: LinearLayoutManager?, dx: Int) {
        // TODO: 10/8/2016
    }

    fun setOrientation(orientation: Int) {
        this.orientation = orientation
    }

    companion object {

        private val TAG = "LinearPaginationDetecto"
        val VERTICAL = LinearLayout.VERTICAL
        val HORIZONTAL = LinearLayout.HORIZONTAL

        private val LOAD_MORE_CHECK_RATE_LIMIT: Long = 300
        private val DEFAULT_LOAD_MORE_BOUND = 20
    }
}
