package com.under9.android.lib.bottomsheet

import android.graphics.Color
import android.graphics.PorterDuff
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.RecyclerView
import com.under9.android.lib.bottomsheet.BottomSheet.ITEM_TYPE_ACCOUNT
import com.under9.android.lib.bottomsheet.BottomSheet.ITEM_TYPE_BADGE
import com.under9.android.lib.bottomsheet.BottomSheet.ITEM_TYPE_COLORIZE
import com.under9.android.lib.bottomsheet.BottomSheet.ITEM_TYPE_DIVIDER
import com.under9.android.lib.bottomsheet.BottomSheet.ITEM_TYPE_HEADER
import com.under9.android.lib.bottomsheet.BottomSheet.ITEM_TYPE_MENU_CHECKED
import com.under9.android.lib.bottomsheet.BottomSheet.ITEM_TYPE_SIMPLE
import com.under9.android.lib.bottomsheet.BottomSheet.ITEM_TYPE_SWITCH
import com.under9.android.lib.bottomsheet.data.BottomSheetModel
import com.under9.android.lib.util.UiUtil
import com.under9.android.lib.util.dpToPx
import com.under9.android.lib.widget.R
import com.under9.android.lib.widget.applyTintColor
import com.under9.android.lib.widget.text.TextDrawable
import kotlinx.android.synthetic.main.view_header_item_text.view.*
import kotlinx.android.synthetic.main.view_item_bottom_sheet_account.view.*
import kotlinx.android.synthetic.main.view_item_bottom_sheet_colorize.view.*
import kotlinx.android.synthetic.main.view_item_bottom_sheet_switch.view.*
import kotlinx.android.synthetic.main.view_item_bottom_sheet_switch.view.featureLockIcon
import kotlinx.android.synthetic.main.view_item_simple_bottom_sheet.view.menuIcon
import kotlinx.android.synthetic.main.view_item_simple_bottom_sheet.view.menuTitle


class BottomSheetAdapter(private val items: MutableList<BottomSheetModel>,
                         private val clickListener: BottomSheetClickListener,
                         private val dismissListener: DismissListener) : RecyclerView.Adapter<RecyclerView.ViewHolder>() {

    fun refreshItems(items: List<BottomSheetModel>) {
        this.items.clear()
        this.items.addAll(items)
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        val inflater = LayoutInflater.from(parent.context)
        return when (viewType) {
            ITEM_TYPE_ACCOUNT -> AccountItemView(inflater.inflate(R.layout.view_item_bottom_sheet_account, parent, false), clickListener, dismissListener)
            ITEM_TYPE_SWITCH -> SwitchItemView(inflater.inflate(R.layout.view_item_bottom_sheet_switch, parent, false), clickListener, dismissListener)
            ITEM_TYPE_BADGE -> BadgeItemView(inflater.inflate(R.layout.view_item_bottom_sheet_badge, parent, false), clickListener, dismissListener)
            ITEM_TYPE_SIMPLE -> SimpleViewHolder(inflater.inflate(R.layout.view_item_simple_bottom_sheet, parent, false), clickListener, dismissListener)
            ITEM_TYPE_MENU_CHECKED -> CheckedView(inflater.inflate(R.layout.view_item_bottom_sheet_checked, parent, false), clickListener, dismissListener)
            ITEM_TYPE_COLORIZE -> ColorizeView(inflater.inflate(R.layout.view_item_bottom_sheet_colorize, parent, false), clickListener, dismissListener)
            ITEM_TYPE_DIVIDER -> DividerItemViewHolder(inflater.inflate(R.layout.view_item_bottom_sheet_divider, parent, false))
            ITEM_TYPE_HEADER -> HeaderItemViewHolder(inflater.inflate(R.layout.view_header_item_text, parent, false))
            else -> SimpleViewHolder(inflater.inflate(R.layout.view_item_simple_bottom_sheet, parent, false), clickListener, dismissListener)
        }
    }

    override fun getItemCount() = items.size

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        when (getItemViewType(position)) {
            ITEM_TYPE_DIVIDER -> return
            ITEM_TYPE_HEADER -> return (holder as HeaderItemViewHolder).bind(items[position], position)
            else -> (holder as BaseViewHolder).bind(items[position], position)
        }
        holder.itemView.tag = items[position].id
    }


    override fun getItemViewType(position: Int) = items[position].displayType

    abstract class BaseViewHolder(itemView: View,
                                  protected val clickListener: BottomSheetClickListener,
                                  protected val dismissListener: DismissListener) : RecyclerView.ViewHolder(itemView) {

        abstract fun bind(model: BottomSheetModel, pos: Position)
    }

    /**
     * Header divider in [BottomSheet]
     */
    class HeaderItemViewHolder(itemView: View): RecyclerView.ViewHolder(itemView) {
        fun bind(model: BottomSheetModel, pos: Position) {
            with(itemView) {
                widget_headerTitle.text = model.title
                divider.visibility = View.GONE
            }
        }
    }

    /**
     * Item in [BottomSheet]
     */
    open class SimpleViewHolder(itemView: View,
                                clickListener: BottomSheetClickListener,
                                dismissListener: DismissListener,
                                private val dismissible: Boolean = true) : BaseViewHolder(itemView, clickListener, dismissListener) {

        override fun bind(model: BottomSheetModel, pos: Position) {
            val title = model.title
            val icon = model.iconRes
            val iconColor = model.additionalIconColor

            with(itemView) {
                menuTitle.text = title

                setBackgroundColor(UiUtil.themeAttributeToColor(R.attr.under9_themeForeground, context, -1))
                menuTitle.setTextColor(UiUtil.themeAttributeToColor(R.attr.under9_themeTextColorPrimary, context, -1))

                when (model.iconType) {
                    BottomSheet.ICON_TYPE_IMAGE -> {
                        menuIcon.apply {
                            visibility = View.VISIBLE
                            setImageDrawable(ContextCompat.getDrawable(context, icon))
                            if (iconColor == -1) {
                                menuIcon.applyTintColor(UiUtil.themeAttributeToColor(R.attr.under9_themeIconColor, context, -1))
                            } else {
                                menuIcon.applyTintColor(ContextCompat.getColor(context, iconColor))
                            }
                        }
                    }

                    BottomSheet.ICON_TYPE_TEXT_EMOJI -> {
                        menuIcon.apply {
                            visibility = View.VISIBLE
                            setImageDrawable(
                                    TextDrawable.builder()
                                            .beginConfig()
                                            .fontSize(16.dpToPx(context))
                                            .endConfig()
                                            .buildRect(model.iconString ?: "", Color.TRANSPARENT)
                            )
                        }
                    }

                    BottomSheet.ICON_TYPE_NONE -> {
                        menuIcon.visibility = View.GONE
                        (menuTitle.layoutParams as ConstraintLayout.LayoutParams).apply {
                            marginStart = UiUtil.dpToPx(itemView.context, 16)
                            leftMargin = UiUtil.dpToPx(itemView.context, 16)
                        }
                    }
                }

                featureLockIcon?.let {
                    it.visibility = if (model.showFeatureLock) View.VISIBLE else View.GONE
                }

                setOnClickListener {
                    clickListener(pos, model.id)
                    if (dismissible) dismissListener()
                }
            }
        }
    }


    class AccountItemView(itemView: View,
                          clickListener: BottomSheetClickListener,
                          dismissListener: DismissListener) : BaseViewHolder(itemView, clickListener, dismissListener) {

        override fun bind(model: BottomSheetModel, pos: Position) {
            val avatarUri = model.additionalStringInfo!!
            val isPro = model.additionalBooleanInfo
            val username = model.title
            val greyProBadge = model.additionalIconColor != -1

            with(itemView) {
                accountAvatar.setImageURI(avatarUri)
                accountAvatar.isActive = model.showIndicator!! && avatarUri.isNotBlank()
                tvUsername.text = username
                tvUsername.setTextColor(UiUtil.themeAttributeToColor(R.attr.under9_themeTextColorPrimary, context, -1))

                setBackgroundColor(UiUtil.themeAttributeToColor(R.attr.under9_themeForeground, context, -1))
                isPro?.let {
                    proBadge.visibility = View.VISIBLE
                    if (greyProBadge) {
                        proBadge.toGreyBadge()
                    }
                    proBadge.updateProBadge(it)
                } ?: run {
                    proBadge.visibility = View.GONE
                }

                setOnClickListener {
                    clickListener(pos, model.id)
                    dismissListener()
                }
            }
        }
    }

    class BadgeItemView(itemView: View,
                        clickListener: BottomSheetClickListener,
                        dismissListener: DismissListener) : SimpleViewHolder(itemView, clickListener, dismissListener, dismissible = true) {

        override fun bind(model: BottomSheetModel, pos: Position) {
            super.bind(model, pos)
            val isPro = model.additionalBooleanInfo
            isPro?.let {
                itemView.proBadge.visibility = View.VISIBLE
                itemView.proBadge.updateProBadge(it)
            } ?: run {
                itemView.proBadge.visibility = View.GONE
            }
        }
    }

    class SwitchItemView(itemView: View,
                         clickListener: BottomSheetClickListener,
                         dismissListener: DismissListener) : SimpleViewHolder(itemView, clickListener, dismissListener, dismissible = false) {

        override fun bind(model: BottomSheetModel, pos: Position) {
            super.bind(model, pos)
            val isChecked = model.additionalBooleanInfo!!
            itemView.actionSwitch.isChecked = isChecked
        }
    }

    class CheckedView(itemView: View,
                      clickListener: BottomSheetClickListener,
                      dismissListener: DismissListener) : SimpleViewHolder(itemView, clickListener, dismissListener, dismissible = true)

    /**
     * additionalIntInfo: color int
     * additionalBooleanInfo: checking the current color is default or not, if default, use the R.drawable.color_circle, else change the tint color
     * */
    class ColorizeView(itemView: View,
                       clickListener: BottomSheetClickListener,
                       dismissListener: DismissListener) : SimpleViewHolder(itemView, clickListener, dismissListener, dismissible = true) {

        override fun bind(model: BottomSheetModel, pos: Position) {
            super.bind(model, pos)
            // is default color
            if (model.additionalBooleanInfo == true) {
                itemView.colorView.background = itemView.context.getDrawable(R.drawable.color_circle)
            } else {
                itemView.colorView.background.setColorFilter(model.additionalIntInfo, PorterDuff.Mode.SRC_ATOP)
            }
        }
    }

    class DividerItemViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView)
}