package com.under9.android.lib.bottomsheet

import android.app.Dialog
import android.os.Build
import android.os.Bundle
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.under9.android.lib.behavior.BedModeController
import com.under9.android.lib.bottomsheet.data.BottomSheetMenuItems
import com.under9.android.lib.bottomsheet.data.BottomSheetModel
import com.under9.android.lib.util.UiUtil
import com.under9.android.lib.view.rootview.GagFrameLayout
import com.under9.android.lib.widget.R

class GagBottomSheetDialogFragment : StyledBottomSheetDialogFragment() {

    private lateinit var items: MutableList<BottomSheetModel>
    private lateinit var bottomSheetAdapter: BottomSheetAdapter
    private var listener: BottomSheetClickListener = { _: Position, _: MenuItemId -> }
    private var showListener: BottomSheetShowDialogListener? = null
    lateinit var rv: RecyclerView
    private lateinit var rootLayout: View

    companion object {
        const val KEY_MENU_ITEMS = "menu_items"
        const val KEY_BED_MODE_ENABLED = "bed_mode_enabled"

        const val FRAG_TAG = "more_action"

        @JvmStatic
        fun newInstance(items: BottomSheetMenuItems, isEnabledBedMode: Boolean): GagBottomSheetDialogFragment {
            val args = Bundle()
            args.putParcelable(KEY_MENU_ITEMS, items)
            args.putBoolean(KEY_BED_MODE_ENABLED, isEnabledBedMode)
            val frag = GagBottomSheetDialogFragment()
            frag.arguments = args
            return frag
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        arguments?.let { it ->
            it.getParcelable<BottomSheetMenuItems>(KEY_MENU_ITEMS)?.let { menuItems ->
                this.items = ArrayList(menuItems.items)
            }
            isEnabledBedModel = it.getBoolean(KEY_BED_MODE_ENABLED, false)
        }
    }

    fun setBottomSheetClickListener(listener: BottomSheetClickListener) {
        this.listener = listener
    }

    fun setShowListener(showListener: BottomSheetShowDialogListener) {
        this.showListener = showListener
    }

    /**
     * To refresh bottom sheet menu's items
     */
    fun refreshAdapterItems(items: List<BottomSheetModel>) {
        if (::bottomSheetAdapter.isInitialized) {
            this.items.clear()
            this.items.addAll(items)
            bottomSheetAdapter.refreshItems(items)
            bottomSheetAdapter.notifyDataSetChanged()
        }
    }

    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
        val dialog = super.onCreateDialog(savedInstanceState)

        val matchAllParentParam = FrameLayout.LayoutParams(ViewGroup.LayoutParams(FrameLayout.LayoutParams.MATCH_PARENT, FrameLayout.LayoutParams.MATCH_PARENT))

        rootLayout = GagFrameLayout(context!!).apply {
            // Currently, just apply the change of dark theme for the version >= LOLLIPOP, because it make codes very messy if handle pre-lollipop here,
            // And it will drop pre-lollipop later, so TODO: clean up when dropped pre-lollipop support
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                setBackgroundColor(UiUtil.themeAttributeToColor(R.attr.under9_themeForeground, context, -1))
            }
            layoutParams = matchAllParentParam
            id = R.id.more_menu_button_sheet
        }

        bottomSheetAdapter = BottomSheetAdapter(items, listener, {
            dialog.dismiss()
        })
        rv = RecyclerView(context!!).apply {
            layoutParams =  FrameLayout.LayoutParams(ViewGroup.LayoutParams(FrameLayout.LayoutParams.MATCH_PARENT, FrameLayout.LayoutParams.MATCH_PARENT)).apply {
                bottomMargin = context!!.resources.getDimension(R.dimen.space8).toInt()
            }
            id = R.id.more_menu_recyclerview
            layoutManager = LinearLayoutManager(context)
            this.adapter = bottomSheetAdapter
            hasFixedSize()
        }

        (rootLayout as GagFrameLayout).addView(rv)
        dialog.setContentView(rootLayout)
        if (isEnabledBedModel) {
            BedModeController(context!!, false, false).apply {
                registerCanvasView(rootLayout as GagFrameLayout)
                disableChangeNavColor()
                enableBedMode()
            }
        }
        dialog.setOnShowListener { showListener?.invoke(dialog) }
        return dialog
    }

    object SheetHelper {

        @JvmStatic
        fun findRow(d: Dialog, rowId: Int): View? {
            val rv = d.findViewById<RecyclerView>(R.id.more_menu_recyclerview)
            if (rv != null && rv.layoutManager != null) {
                val totalChildCount = rv.layoutManager!!.childCount
                for (i in 0 until totalChildCount) {
                    val viewHolder =  rv.getChildViewHolder(rv.getChildAt(i))
                    if (viewHolder.itemView.tag != null && (viewHolder.itemView.tag as Int) == rowId) {
                        return viewHolder.itemView
                    }
                }
            }
            return null
        }
    }
}

typealias BottomSheetClickListener = ((Position), (MenuItemId)) -> Unit
typealias BottomSheetShowDialogListener = (Dialog) -> Unit
typealias DismissListener = () -> Unit
typealias MenuItemId = Int
typealias Position = Int