package com.under9.android.lib.util

import android.content.ContentResolver
import android.content.ContentValues
import android.content.Context
import android.database.Cursor
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.net.Uri
import android.os.Build
import android.os.Environment
import android.provider.MediaStore
import com.under9.android.lib.util.file.SourceFileController
import timber.log.Timber
import java.io.ByteArrayOutputStream
import java.io.File
import java.io.FileNotFoundException
import java.io.InputStream

/**
 * Created by raymond on 12/5/2017.
 */

object ContentProviderUtil {

    @JvmStatic fun insertFileProviderMedia(context: Context,
                                fileProviderUri: Uri,
                                sourceFileController: SourceFileController): Uri? {
        val name = "tmp_" + System.currentTimeMillis()
        val f = File(sourceFileController.uploadDir(context), name)

        return try {
            val imageUrl = insertMedia(context.contentResolver, fileProviderUri.toString(), f.absolutePath, name, null)
            Uri.parse(imageUrl)
        } catch (e: Exception) {
            Timber.e(e)
            null
        }
    }

    /**
     * Ref {@link [MediaStore.Images.Media.insertImage]
     * @return contentURI
     */
    @Throws(FileNotFoundException::class)
    @JvmStatic fun insertMedia(cr: ContentResolver,
                               sourcePath: String?,
                               targetPath: String,
                               title: String,
                               description: String?): String? {
        val rawMimeType = cr.getType(Uri.parse(sourcePath))
                ?: return null

        // If image is coming from camera, it will have the type "application/octet-stream"
        val mimeType = if (rawMimeType == "application/octet-stream") {
            "image/*"
        } else {
            rawMimeType
        }

        val values = ContentValues().apply {
            put(MediaStore.Images.Media.TITLE, title)
            put(MediaStore.Images.Media.DESCRIPTION, description)
            put(MediaStore.Images.Media.MIME_TYPE, mimeType)

            if (Build.VERSION.SDK_INT <= 28) {
                put(MediaStore.Images.Media.DATA, targetPath)
            } else {
                put(MediaStore.Images.Media.RELATIVE_PATH, "${Environment.DIRECTORY_PICTURES}/$title")
                put(MediaStore.Images.Media.IS_PENDING, 1)
            }
        }

        /**
         * @link https://developer.android.com/training/data-storage/shared/media#add-item
         */
        val volumeName = if (Build.VERSION.SDK_INT <= 28) MediaStore.VOLUME_EXTERNAL else MediaStore.VOLUME_EXTERNAL_PRIMARY
        val collection = MediaStore.Images.Media.getContentUri(volumeName)
        var url: Uri? = null
        var stringUrl: String? = null    /* value to be returned */
        var inputStream: InputStream? = null

        Timber.d("mimeType=$rawMimeType, targetPath=$targetPath, sourcePath=$sourcePath")

        try {
            inputStream = cr.openInputStream(Uri.parse(sourcePath))
            url = cr.insert(collection, values)

            when (rawMimeType) {
                "image/gif" -> {
                    val os = cr.openOutputStream(url!!, "w")
                    try {
                        val buffer = ByteArray(1024)
                        var len: Int
                        while ((inputStream!!.read(buffer).also { len = it }) != -1) {
                            os?.write(buffer, 0, len)
                        }
                    } finally {
                        os?.flush()
                        os?.close()
                    }
                }

                else -> {
                    val source = BitmapFactory.decodeStream(inputStream)
                    if (source != null) {
                        val imageOut = cr.openOutputStream(url!!)
                        val bytes = ByteArrayOutputStream()
                        source.compress(Bitmap.CompressFormat.JPEG, 100, bytes)
                        try {
                            source.compress(Bitmap.CompressFormat.JPEG, 50, imageOut)
                        } finally {
                            imageOut!!.close()
                        }
                    } else {
                        if (url != null) {
                            cr.delete(url, null, null)
                            url = null
                        }
                    }
                }
            }

        } catch (e: Exception) {
            Timber.e(e, "Failed to insert image")
            if (url != null) {
                cr.delete(url, null, null)
                url = null
            }
        } finally {
            inputStream?.close()
        }


        if (url != null) {
            stringUrl = url.toString()
        }

        return stringUrl
    }

    /**
     * Get the value of the data column for this Uri. This is useful for
     * MediaStore Uris, and other file-based ContentProviders.
     *
     * @param context The context.
     * @param uri The Uri to query.
     * @param selection (Optional) Filter used in the query.
     * @param selectionArgs (Optional) Selection arguments used in the query.
     * @return The value of the _data column, which is typically a file path.
     */
    @JvmStatic fun getDataColumn(context: Context,
                      uri: Uri,
                      selection: String?,
                      selectionArgs: Array<String>?): String? {

        var cursor: Cursor? = null
        val column = MediaStore.Images.ImageColumns.DATA
        val projection = arrayOf(column)

        try {
            cursor = context.contentResolver.query(uri, projection, selection, selectionArgs, null)
            if (cursor != null && cursor.moveToFirst()) {
                val columnIndex = cursor.getColumnIndexOrThrow(column)
                return cursor.getString(columnIndex)
            }
        } catch (e: Exception) {
            Timber.e(e)
        } finally {
            cursor?.close()
        }
        return null
    }

    /**
     * @param uri The Uri to check.
     * @return Whether the Uri authority is ExternalStorageProvider.
     */
    @JvmStatic fun isExternalStorageDocument(uri: Uri): Boolean {
        return "com.android.externalstorage.documents" == uri.authority
    }

    /**
     * @param uri The Uri to check.
     * @return Whether the Uri authority is DownloadsProvider.
     */
    @JvmStatic fun isDownloadsDocument(uri: Uri): Boolean {
        return "com.android.providers.downloads.documents" == uri.authority
    }

    /**
     * @param uri The Uri to check.
     * @return Whether the Uri authority is MediaProvider.
     */
    @JvmStatic fun isMediaDocument(uri: Uri): Boolean {
        return "com.android.providers.media.documents" == uri.authority
    }

    /**
     *
     * @param uri
     * @return
     */
    @JvmStatic fun isGooglePhotosUri(uri: Uri): Boolean {
        return "com.google.android.apps.photos.content" == uri.authority
    }

    /**
     *
     * @param uri
     * @return
     */
    @JvmStatic fun isGooglePlayPhotosUri(uri: Uri): Boolean {
        return "com.google.android.apps.photos.contentprovider" == uri.authority
    }

    /**
     * Gboard
     * @param uri
     */
    @JvmStatic fun isGboardUri(uri: Uri): Boolean {
        return "com.google.android.inputmethod.latin.fileprovider" == uri.authority
    }
}
