// from iosched 2012 JSONHandler
package com.under9.android.lib.util.file

import android.annotation.TargetApi
import android.content.ContentUris
import android.content.Context
import android.content.Intent
import android.graphics.Bitmap
import android.net.Uri
import android.os.Build
import android.os.Environment
import android.provider.MediaStore
import android.util.Log
import androidx.core.content.FileProvider
import timber.log.Timber
import java.io.*
import java.util.*
import kotlin.math.min

object FileUtil {
    private const val TAG = "FileUtil"
    private const val DEBUG = false
    @JvmStatic
    @Throws(FileNotFoundException::class)
    fun saveBitmap(bmp: Bitmap, format: Bitmap.CompressFormat?, quality: Int, path: String?) {
        val f = File(path)
        val fos = FileOutputStream(f)
        bmp.compress(format, quality, fos)
        try {
            fos.close()
        } catch (e: IOException) {
        }
    }

    @Throws(IOException::class)
    fun readFile(`is`: InputStream): String {
        val writer: Writer = StringWriter()
        val buffer = CharArray(1024)
        try {
            val reader: Reader = BufferedReader(InputStreamReader(`is`, "UTF-8"))
            var n: Int
            while (reader.read(buffer).also { n = it } != -1) {
                writer.write(buffer, 0, n)
            }
        } finally {
            `is`.close()
        }
        return writer.toString()
    }

    fun inputStreamToBytes(`is`: InputStream): ByteArray {
        val bos = ByteArrayOutputStream()
        val b = ByteArray(1024)
        var bytesRead: Int
        try {
            while (`is`.read(b).also { bytesRead = it } != -1) {
                bos.write(b, 0, bytesRead)
            }
        } catch (e: IOException) {
        }
        return bos.toByteArray()
    }

    @JvmStatic
    @Throws(IOException::class)
    fun copy(src: File?, dst: File?) {
        val `in`: InputStream = FileInputStream(src)
        val out: OutputStream = FileOutputStream(dst)
        // Transfer bytes from in to out
        val buf = ByteArray(1024)
        var len: Int
        while (`in`.read(buf).also { len = it } > 0) {
            out.write(buf, 0, len)
        }
        `in`.close()
        out.close()
    }

    @JvmStatic
    fun stringToFilename(s: String): String {
        return s.replace(" ".toRegex(), "-").replace("[^0-9a-zA-Z\\-]".toRegex(), "")
    }

    fun deleteFile(path: String?) {
        File(path).delete()
    }

    fun getExtension(s: String): String {
        val p = s.lastIndexOf(".")
        return if (p == -1) {
            s
        } else {
            s.substring(p + 1, s.length)
        }
    }

    /**
     * Delete by string path
     * @param path
     * @return
     */
    fun deleteDirectory(path: String?): Boolean {
        return if (path == null) false else deleteDirectory(File(path), null)
    }

    /**
     *
     * @param file
     * @return
     */
    @JvmStatic fun deleteDirectory(file: File): Boolean {
        return deleteDirectory(file, null)
    }

    /**
     *
     * @param path
     * @param filter Filter to specify which file should be filtered out during the delete process
     * @return
     */
    @JvmStatic
    fun deleteDirectory(path: File, filter: FileFilter?): Boolean {
        if (path.exists()) {
            val files: Array<File>?
            files = if (filter == null) {
                path.listFiles()
            } else {
                path.listFiles(filter)
            }
            if (files == null) {
                return true
            }
            for (i in files.indices) {
                if (files[i].isDirectory) {
                    deleteDirectory(files[i], filter)
                } else {
                    files[i].delete()
                }
            }
        }
        return path.delete()
    }

    @JvmStatic
    fun mkdirs(path: String?) {
        try {
            val dir = File(path)
            dir.mkdirs()
        } catch (e: Exception) {
        }
    }

    // Unlimited
    val cameraPhotos: List<String>
        get() = getCameraPhotos(-1) // Unlimited

    fun getCameraPhotos(limit: Int): List<String> {
        val filePaths = ArrayList<String>()
        val fileFilter = ImageFileFilter()
        return try {
            val f = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DCIM)
            val files = f.listFiles()
            for (i in files.indices) {
                if (files[i].isDirectory && files[i].name == ".thumbnails") {
                    continue
                }
                if (files[i].isDirectory) {
                    val cameraImages = files[i].listFiles(fileFilter)
                    Arrays.sort(cameraImages) { lhs, rhs -> java.lang.Long.valueOf(rhs.lastModified()).compareTo(lhs.lastModified()) }
                    for (j in cameraImages.indices) {
                        filePaths.add(cameraImages[j].absolutePath)
                        if (limit != -1 && limit < filePaths.size) {
                            return filePaths
                        }
                    }
                }
            }
            filePaths
        } catch (e: Exception) {
            filePaths
        }
    }

    /**
     * Get image with callback
     * @param limit
     * @param callback
     * @return
     */
    fun getCameraPhotos(limit: Int, callback: AddImageCallback?): List<String> {
        val filePaths = ArrayList<String>()
        val fileFilter = ImageFileFilter()
        return try {
            val f = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DCIM)
            val files = f.listFiles()
            for (i in files.indices) {
                if (files[i].isDirectory && files[i].name == ".thumbnails") {
                    continue
                }
                if (files[i].isDirectory) {
                    val cameraImages = files[i].listFiles(fileFilter)
                    Arrays.sort(cameraImages) { lhs, rhs -> java.lang.Long.valueOf(rhs.lastModified()).compareTo(lhs.lastModified()) }
                    for (j in cameraImages.indices) {
                        val url = cameraImages[j].absolutePath
                        filePaths.add(url)
                        callback?.onImageAdded(url)
                        if (limit != -1 && limit < filePaths.size) {
                            return filePaths
                        }
                    }
                }
            }
            filePaths
        } catch (e: Exception) {
            filePaths
        }
    }

    private data class Media(val uri: Uri,
                             val name: String,
                             val duration: Long,
                             val mimeType: String,
                             val dateAdded: Long): Comparable<Media> {
        override fun compareTo(other: Media): Int = this.dateAdded.compareTo(other.dateAdded)
    }

    @JvmStatic fun getMediaByContentResolver2(context: Context, limit: Int, callback: LoadMediaCallback?) {
        val total = ArrayList<Media>()
        val imageMedia = ArrayList<Media>()
        val videoMedia = ArrayList<Media>()

        try {
            val cr = context.applicationContext.contentResolver
            val sortOrderImage = "${MediaStore.Images.Media.DATE_ADDED} DESC LIMIT $limit"
            val sortOrderVideo = "${MediaStore.Video.Media.DATE_ADDED} DESC LIMIT $limit"

            val projectionImage = arrayOf(
                    MediaStore.Images.Media._ID,
                    MediaStore.Images.Media.DISPLAY_NAME,
                    MediaStore.Images.Media.MIME_TYPE,
                    MediaStore.Images.Media.DATE_ADDED
            )

            val projectionVideo = arrayOf(
                    MediaStore.Video.Media._ID,
                    MediaStore.Video.Media.DISPLAY_NAME,
                    MediaStore.Video.Media.MIME_TYPE,
                    MediaStore.Video.Media.DATE_ADDED,
                    MediaStore.Video.Media.DURATION
            )

            cr.query(MediaStore.Images.Media.EXTERNAL_CONTENT_URI,
                    projectionImage, null,
                    null, sortOrderImage)?.use { cursor ->
                if (cursor.moveToFirst()) {
                    do {
                        val idColumn = cursor.getColumnIndexOrThrow(MediaStore.Images.Media._ID)
                        val nameColumn = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.DISPLAY_NAME)
                        val mimeTypeColumn = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.MIME_TYPE)
                        val dateAddedColumn = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.DATE_ADDED)

                        val id = cursor.getLong(idColumn)
                        val name = cursor.getString(nameColumn)
                        val mimeType = cursor.getString(mimeTypeColumn)
                        val dateAdded = cursor.getLong(dateAddedColumn)
                        val contentUri: Uri = ContentUris.withAppendedId(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, id)
                        imageMedia += Media(contentUri, name, 0, mimeType, dateAdded)

                    } while ((cursor.moveToNext()) && imageMedia.size < limit)
                }
            }

            cr.query(MediaStore.Video.Media.EXTERNAL_CONTENT_URI,
                    projectionVideo, null,
                    null, sortOrderVideo)?.use { cursor ->
                if (cursor.moveToFirst()) {
                    do {
                        val idColumn = cursor.getColumnIndexOrThrow(MediaStore.Video.Media._ID)
                        val nameColumn = cursor.getColumnIndexOrThrow(MediaStore.Video.Media.DISPLAY_NAME)
                        val mimeTypeColumn = cursor.getColumnIndexOrThrow(MediaStore.Video.Media.MIME_TYPE)
                        val dateAddedColumn = cursor.getColumnIndexOrThrow(MediaStore.Video.Media.DATE_ADDED)
                        val durationColumn = cursor.getColumnIndexOrThrow(MediaStore.Video.Media.DURATION)

                        val id = cursor.getLong(idColumn)
                        val name = cursor.getString(nameColumn)
                        val mimeType = cursor.getString(mimeTypeColumn)
                        val dateAdded = cursor.getLong(dateAddedColumn)
                        val duration = cursor.getLong(durationColumn)
                        val contentUri: Uri = ContentUris.withAppendedId(MediaStore.Video.Media.EXTERNAL_CONTENT_URI, id)
                        videoMedia += Media(contentUri, name, duration / 1000, mimeType, dateAdded)
                    } while ((cursor.moveToNext()) && videoMedia.size < limit)
                }

            }

            total.addAll(imageMedia)
            total.addAll(videoMedia)
            total.sortDescending()
            total.subList(0, min(limit, total.size)).forEach { media ->
                val (uri, name, duration, mimeType) = media
                Timber.d("$media, date=${Date(media.dateAdded)}")
                callback?.onMediaLoaded(uri.toString(), duration, mimeType, name)
            }

        } catch (e: Exception) {
            Timber.e(e)
        }
    }

    @JvmStatic
    @TargetApi(value = Build.VERSION_CODES.HONEYCOMB)
    @Deprecated("call getMediaByContentResolver2")
    fun getMediaByContentResolver(context: Context, limit: Int, callback: LoadMediaCallback?) {
        val filePaths = ArrayList<String>()
        try {
            val cr = context.contentResolver
            val columns = arrayOf(
                    MediaStore.Files.FileColumns._ID,
                    MediaStore.Files.FileColumns.TITLE,
                    MediaStore.Files.FileColumns.DATA,
                    MediaStore.Files.FileColumns.MIME_TYPE,
                    MediaStore.Files.FileColumns.SIZE)
            val selection = (MediaStore.Files.FileColumns.MEDIA_TYPE + "="
                    + MediaStore.Files.FileColumns.MEDIA_TYPE_IMAGE
                    + " OR "
                    +
                    MediaStore.Files.FileColumns.MEDIA_TYPE + "="
                    + MediaStore.Files.FileColumns.MEDIA_TYPE_VIDEO)
            val cursor = cr.query(MediaStore.Files.getContentUri("external"),
                    columns, selection, null, MediaStore.Files.FileColumns.DATE_ADDED + " DESC")
            val sfc = SourceFileController(true)
            val filesDirPath = sfc.getFileDirectory(context.applicationContext).absolutePath
            val cacheDirPath = sfc.getCacheDirectory(context.applicationContext).absolutePath
            if (cursor!!.moveToFirst()) {
                val dataColumn = cursor.getColumnIndexOrThrow(MediaStore.Files.FileColumns.DATA)
                val mimeTypeColumn = cursor.getColumnIndexOrThrow(MediaStore.Files.FileColumns.MIME_TYPE)
                do {
                    val data = cursor.getString(dataColumn)
                    val mimeType = cursor.getString(mimeTypeColumn)
                    if (!(data.contains(cacheDirPath) || data.contains(filesDirPath))) {
                        filePaths.add(data)
                        callback?.onMediaLoaded(data, 0, mimeType, "")
                    }
                } while (cursor.moveToNext() && filePaths.size < limit)
            }
        } catch (e: Exception) {
            Timber.e(e)
        }
    }

    /**
     * Get folder size.
     * @param directory
     * @return
     */
    fun getFolderSize(directory: File): Long {
        var length: Long = 0
        try {
            val files = directory.listFiles()
            if (files != null) {
                for (file in files) {
                    length += if (file.isFile) {
                        file.length()
                    } else {
                        getFolderSize(file)
                    }
                }
            }
        } catch (e: Exception) {
            Log.w(TAG, "getFolderSize: ", e)
        } finally {
            return length
        }
    }

    /**
     * Given a path, trigger media scanning to make the media available in gallery
     * @param context
     * @param path
     */
    @JvmStatic
    fun triggerMediaScanner(context: Context, path: String?) {
        var path = path
        try {
            if (path == null) {
                path = "file://" + Environment.getExternalStorageDirectory()
            }
            val mediaScanIntent = Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE)
            val f = File(path)
            val contentUri = Uri.fromFile(f)
            mediaScanIntent.data = contentUri
            context.sendBroadcast(mediaScanIntent)
        } catch (e: Exception) {
            Log.e(TAG, e.message, e)
        }
    }

    @JvmStatic
    fun getVersionCompatContentUri(context: Context, file: File): Uri {
        return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            FileProvider.getUriForFile(context, context.applicationContext.packageName + ".fileprovider", file)
        } else Uri.fromFile(file)
    }

    /**
     *
     * @param bytes
     * @param precision
     * @return
     */
    @JvmOverloads
    fun simpleHumanReadableByteCount(bytes: Long, precision: Int = 0): String {
        val unit = 1024
        if (bytes < unit) return "$bytes B"
        val exp = (Math.log(bytes.toDouble()) / Math.log(unit.toDouble())).toInt()
        val pre = Character.toString("KMGTPE"[exp - 1])
        val precisionStr = "%." + precision + "f"
        return String.format("$precisionStr %sB", bytes / Math.pow(unit.toDouble(), exp.toDouble()), pre)
    }

    @Deprecated("Please use {@link LoadMediaCallback}")
    interface AddImageCallback {
        fun onImageAdded(url: String?)
    }

    interface LoadMediaCallback {
        fun onMediaLoaded(url: String?, duration: Long, mimeType: String?, name: String)
    }
}