package com.under9.android.lib.widget

import android.app.Activity
import android.content.Context
import android.graphics.Color
import android.os.Build
import android.view.View
import android.view.Window
import com.under9.android.lib.util.UiUtil

class SystemUIColorRestorer(private val context: Context,
                            private val targetWindow: Window) {

    private val originalSystemUiVisibility: Int
    private val decorView: View
    private val hasLightStatusBar: Boolean
    private val hasLightNavBar: Boolean

    private var originalStatusBarColor: Int? = null
    private var originalNavBarColor: Int? = null

    // cached, use for update status bar again without specify color and type
    private var cachedColor: Int = UiUtil.themeAttributeToColor(R.attr.under9_themeStatusBarColor, context, -1)
    private var colorizeStateBar: Boolean = false
    private var colorizeNavBar: Boolean = false

    init {
        with((context as Activity)) {
            originalSystemUiVisibility = targetWindow.decorView.systemUiVisibility
            decorView = targetWindow.decorView
            hasLightStatusBar = originalSystemUiVisibility and View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR == View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
            hasLightNavBar = originalSystemUiVisibility and View.SYSTEM_UI_FLAG_LIGHT_NAVIGATION_BAR == View.SYSTEM_UI_FLAG_LIGHT_NAVIGATION_BAR

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {

                originalStatusBarColor = targetWindow.statusBarColor
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O_MR1) {
                    originalNavBarColor = targetWindow.navigationBarColor
                }
            }
        }
    }

    fun colorizeUI(color: Int, colorizeStateBar: Boolean = true, colorizeNavBar: Boolean = true) {
        cachedColor = color
        this.colorizeStateBar = colorizeStateBar
        this.colorizeNavBar = colorizeNavBar
        resumeColorizeUI()
    }

    fun resumeColorizeUI() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            if (colorizeStateBar) {
                targetWindow.statusBarColor = cachedColor
            }

            if (colorizeNavBar) {
                targetWindow.navigationBarColor = cachedColor
            }

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O_MR1) {
                if (hasLightNavBar && hasLightStatusBar) {
                    if (cachedColor != UiUtil.themeAttributeToColor(R.attr.under9_themeStatusBarColor, context, -1)) {
                        decorView.systemUiVisibility = 0 or View.SYSTEM_UI_FLAG_LIGHT_NAVIGATION_BAR
                    } else {
                        decorView.systemUiVisibility = View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR or View.SYSTEM_UI_FLAG_LIGHT_NAVIGATION_BAR
                    }
                }
            } else if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                if (hasLightStatusBar) {
                    if (cachedColor != UiUtil.themeAttributeToColor(R.attr.under9_themeStatusBarColor, context, -1)) {
                        decorView.systemUiVisibility = 0
                    } else {
                        decorView.systemUiVisibility = View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
                    }
                }
            }

        }
    }

    fun darkenUI() {
        cachedColor = Color.BLACK
        this.colorizeStateBar = true
        this.colorizeNavBar = true
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            targetWindow.statusBarColor = Color.BLACK
            targetWindow.navigationBarColor = Color.BLACK
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            decorView.systemUiVisibility = 0
        }
    }

    /**
     * Previous state of status bar/ nav bar color
     * */
    fun restoreOriginalColor() {
        val activity = context as Activity

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            originalStatusBarColor?.let {
                targetWindow.statusBarColor = it
            } ?: run {
                targetWindow.statusBarColor = UiUtil.themeAttributeToColor(R.attr.under9_themeStatusBarColor, context, -1)
            }
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O_MR1) {
                originalNavBarColor?.let {
                    targetWindow.navigationBarColor = it
                } ?: run {
                    targetWindow.navigationBarColor = UiUtil.themeAttributeToColor(R.attr.under9_themeNavigationBarColor, context, -1)
                }

                if (hasLightNavBar && hasLightStatusBar) {
                    decorView.systemUiVisibility = View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR or View.SYSTEM_UI_FLAG_LIGHT_NAVIGATION_BAR
                }
            } else if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                if (hasLightStatusBar) {
                    decorView.systemUiVisibility = View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
                }
            }
        }
    }

}